Package: twscrapeR
Type: Package
Title: Twitter/X Scraping via Python's 'twscrape' Library
Version: 0.1.3
Date: 2026-05-23
Authors@R: c(
    person("Agustin", "Nieto", email = "agustin.nieto77@gmail.com", role = c("aut", "cre")),
    person("Claude", "AI", role = "ctb", comment = "Package development assistance")
    )
Description: A comprehensive R interface to Python's 'twscrape' library for scraping
    Twitter/X data. This package uses 'reticulate' to provide a seamless R interface to
    the fully functional Python 'twscrape' library. Supports searching tweets, user
    timelines, followers, and more, with built-in rate limiting and multi-account
    support. Built on top of 'twscrape' by vladkens
    <https://github.com/vladkens/twscrape> and inspired by 'snscrape' by
    JustAnotherArchivist <https://github.com/JustAnotherArchivist/snscrape>.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 3.6.0)
Imports: reticulate (>= 1.20), cli (>= 3.0.0), jsonlite
Suggests: dplyr, purrr, tibble, testthat (>= 3.0.0)
Config/testthat/edition: 3
RoxygenNote: 7.3.3
URL: https://github.com/agusnieto77/twscrapeR
BugReports: https://github.com/agusnieto77/twscrapeR/issues
NeedsCompilation: no
Packaged: 2026-05-30 02:35:27 UTC; agustin
Author: Agustin Nieto [aut, cre],
  Claude AI [ctb] (Package development assistance)
Maintainer: Agustin Nieto <agustin.nieto77@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-03 13:10:02 UTC
Built: R 4.5.2; ; 2026-06-03 15:54:31 UTC; unix

Package: twscrapeR
Type: Package
Title: Twitter/X Scraping via Python's 'twscrape' Library
Version: 0.1.3
Date: 2026-05-23
Authors@R: c(
    person("Agustin", "Nieto", email = "agustin.nieto77@gmail.com", role = c("aut", "cre")),
    person("Claude", "AI", role = "ctb", comment = "Package development assistance")
    )
Description: A comprehensive R interface to Python's 'twscrape' library for scraping
    Twitter/X data. This package uses 'reticulate' to provide a seamless R interface to
    the fully functional Python 'twscrape' library. Supports searching tweets, user
    timelines, followers, and more, with built-in rate limiting and multi-account
    support. Built on top of 'twscrape' by vladkens
    <https://github.com/vladkens/twscrape> and inspired by 'snscrape' by
    JustAnotherArchivist <https://github.com/JustAnotherArchivist/snscrape>.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 3.6.0)
Imports: reticulate (>= 1.20), cli (>= 3.0.0), jsonlite
Suggests: dplyr, purrr, tibble, testthat (>= 3.0.0)
Config/testthat/edition: 3
RoxygenNote: 7.3.3
URL: https://github.com/agusnieto77/twscrapeR
BugReports: https://github.com/agusnieto77/twscrapeR/issues
NeedsCompilation: no
Packaged: 2026-05-30 02:35:27 UTC; agustin
Author: Agustin Nieto [aut, cre],
  Claude AI [ctb] (Package development assistance)
Maintainer: Agustin Nieto <agustin.nieto77@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-03 13:10:02 UTC
Built: R 4.6.0; ; 2026-06-03 18:13:47 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,twscraper_user)
S3method(print,twscraper_tweet)
S3method(print,twscraper_tweets)
S3method(print,twscraper_user)
S3method(print,twscraper_users)
export(add_account)
export(add_account_from_env)
export(check_setup)
export(delete_account)
export(filter_by_date)
export(filter_by_lang)
export(get_followers)
export(get_following)
export(get_retweeters)
export(get_retweeters_batch)
export(get_user)
export(list_accounts)
export(save_csv)
export(save_json)
export(search_hashtag)
export(search_mentions)
export(search_tweets)
export(setup_twscraper)
export(sort_tweets)
export(to_dataframe)
export(tweet_details)
export(tweet_replies)
export(user_media)
export(user_tweets)
export(user_tweets_and_replies)
export(verified_followers)

# twscrapeR

> **Twitter/X Scraping for R via Python's twscrape**

[![R](https://img.shields.io/badge/R-%3E%3D3.6.0-blue.svg)](https://www.r-project.org/)
[![Python](https://img.shields.io/badge/Python-%3E%3D3.10-green.svg)](https://www.python.org/)
[![License](https://img.shields.io/badge/License-MIT-yellow.svg)](LICENSE)

## 📖 Tabla de Contenidos

- [Características](#-características)
- [Instalación](#-instalación)
- [Inicio Rápido](#-inicio-rápido-3-pasos)
- [Funciones Principales](#-funciones-principales)
- [Referencia Completa de Funciones](#-referencia-completa-de-funciones) ⭐ 26 funciones
- [Análisis de Datos](#-análisis-de-datos)
- [Solución de Problemas](#-solución-de-problemas)
- [Estructura de Datos](#-estructura-de-datos)
- [Agradecimientos](#-agradecimientos)
- [Licencia](#-licencia)

## 🌟 Características

- ✅ **Configuración Guiada**: Verifica Python y `twscrape` sin instalar software automáticamente
- ✅ **Basado en `twscrape`**: Usa la librería Python `twscrape` desde R mediante `reticulate`
- ✅ **Interfaz R Nativa**: Parece R puro, no necesitas saber Python
- ✅ **Multi-cuenta**: Soporta múltiples cuentas con rate limiting automático
- ✅ **26 Funciones**: Funcionalidad completa y simplificada para scraping de Twitter/X
- ✅ **Análisis de Redes**: Followers, following, retweeters, verified followers
- ✅ **Análisis de Conversaciones**: Tweet details, replies, menciones
- ✅ **Conversión Fácil**: to_dataframe() convierte tweets y usuarios a dataframes para análisis en R
- ✅ **Sin HTTP propio en R**: delega scraping, rate limiting y colas a `twscrape`

## 📦 Instalación

```r
# Instalar desde GitHub
library(devtools)
install_github("agusnieto77/twscrapeR")
```

**Requisito importante del entorno Python**: usá `twscrape >= 0.18.0` con Python >= 3.10.
La versión `twscrape 0.17.0` quedó rota para `SearchTimeline` después de cambios en X/Twitter y puede fallar con `IndexError: list index out of range`. Desde `twscrape 0.18.0`, el fix viene upstream y `twscrapeR` ya no necesita aplicar un monkeypatch local.

Instalá o actualizá `twscrape` fuera de las funciones del paquete, por ejemplo en una terminal:

```sh
python -m pip install "twscrape>=0.18.0"
```

## 🚀 Inicio Rápido (3 Pasos)

### Paso 1: Configuración Inicial

```r
library(twscrapeR)

# Configuración guiada (solo verifica el entorno; no instala software)
setup_twscraper()
```

La función `setup_twscraper()` te guiará a través de:
1. Detección de Python >= 3.10
2. Verificación de la librería Python `twscrape`
3. Configuración del entorno

**Si no tenés Python o `twscrape` instalado**, `setup_twscraper()` mostrará instrucciones para instalarlo manualmente fuera de R.

### Paso 2: Agregar Cuenta

Opción recomendada: guardá tus credenciales en `.Renviron` y agregá la cuenta sin escribir secretos en el script.

```r
# En .Renviron
TWS_USERNAME='tu_usuario'
TWS_PASSWORD='tu_password'
TWS_EMAIL='tu@email.com'
TWS_EMAIL_PASSWORD='email_pass'
# Opción A: cookie completa
TWS_COOKIES='auth_token=valor_auth_token; ct0=valor_ct0'

# Opción B: valores separados; add_account_from_env() arma la cookie
TWS_AUTH_TOKEN='valor_auth_token'
TWS_CT0='valor_ct0'

# En R
add_account_from_env()
```

Para varias cuentas podés repetir el esquema con otro prefijo y pasarlo a la función, por ejemplo `TWS2_USERNAME`, `TWS2_PASSWORD`, `TWS2_COOKIES`, etc. y luego `add_account_from_env(prefix = "TWS2_")`.

También podés pasar todo explícitamente:

```r
add_account(
  username = "tu_usuario",
  password = "tu_password",
  email = "tu@email.com",
  email_password = "email_pass",
  cookies = "auth_token=...; ct0=..."
)
```

**Nota IMPORTANTE**: Las cookies son **OBLIGATORIAS** para que la cuenta se active correctamente. Podés pasarlas como `TWS_COOKIES` o como `TWS_AUTH_TOKEN` + `TWS_CT0`.

#### ¿Cómo obtener cookies?

1. Abre https://x.com en tu navegador (con sesión iniciada)
2. Presiona F12 → "Application" → "Cookies" → "https://x.com"
3. Copia los valores de `auth_token` y `ct0`
4. Formato: `"auth_token=valor1; ct0=valor2"`

### Paso 3: ¡Usar!

```r
# Buscar tweets
tweets <- search_tweets("rstats", n = 100)

# Por defecto busca Latest; para tweets destacados/populares usá Top
top_tweets <- search_tweets("rstats", n = 100, product = "Top")

# Convertir a dataframe
df <- to_dataframe(tweets)
View(df)

# Guardar
save_csv(tweets, "tweets_rstats.csv")
```

## 📖 Funciones Principales

### Búsqueda de Tweets

```r
# Búsqueda general
tweets <- search_tweets("machine learning", n = 100)

# Latest es el valor por defecto; Top devuelve resultados destacados/populares
latest <- search_tweets("machine learning", n = 100, product = "Latest")
top <- search_tweets("machine learning", n = 100, product = "Top")

# Por hashtag
tweets <- search_hashtag("rstats", n = 50)
tweets <- search_hashtag("#datascience", n = 100)  # Con o sin #

# Menciones de un usuario
mentions <- search_mentions("hadleywickham", n = 50)

# Búsquedas avanzadas
tweets <- search_tweets("rstats lang:en", n = 100)  # Solo inglés
tweets <- search_tweets("from:hadleywickham", n = 50)    # De un usuario específico
```

### Tweets de Usuario

```r
# Obtener tweets de un usuario
tweets <- user_tweets("hadleywickham", n = 100)

# Ver información del usuario
user <- get_user("hadleywickham")
print(user)
```

### Gestión de Cuentas

```r
# Listar cuentas configuradas
list_accounts()

# Eliminar una cuenta
delete_account("usuario")
```

### Conversión y Exportación

```r
# Convertir a dataframe
df <- to_dataframe(tweets)

# Guardar como CSV
save_csv(tweets, "tweets.csv")

# Guardar como JSON
save_json(tweets, "tweets.json")
```

### Filtrado y Ordenamiento

```r
# Filtrar por idioma
tweets_es <- filter_by_lang(tweets, "es")
tweets_en <- filter_by_lang(tweets, "en")

# Filtrar por fecha
tweets_recent <- filter_by_date(tweets, from = "2025-10-01")
tweets_range <- filter_by_date(tweets, from = "2025-10-01", to = "2025-10-26")

# Ordenar por likes
top_tweets <- sort_tweets(tweets, by = "like_count")

# Ordenar por fecha (más recientes primero)
recent_first <- sort_tweets(tweets, by = "date", decreasing = TRUE)
```

## 📚 Referencia Completa de Funciones

### 1. Configuración y Cuentas (5 funciones)

```r
# Configuración inicial del sistema
setup_twscraper()

# Agregar cuenta - se activa automáticamente con cookies
add_account_from_env()

# O pasar los datos explícitamente
add_account(
  username = "usuario",
  password = "pass",
  email = "email@example.com",
  email_password = "email_pass",
  cookies = "auth_token=...; ct0=..."  # OBLIGATORIO
)

# Listar todas las cuentas configuradas (muestra estado activo/inactivo)
accounts <- list_accounts()

# Eliminar una cuenta
delete_account("usuario")

# Verificar configuración del sistema
check_setup()
```

### 2. Búsqueda de Tweets (3 funciones)

```r
# Búsqueda general de tweets; product acepta "Latest" o "Top"
tweets <- search_tweets("machine learning", n = 100)
top_tweets <- search_tweets("machine learning", n = 100, product = "Top")

# Buscar por hashtag
tweets <- search_hashtag("rstats", n = 50)
tweets <- search_hashtag("#datascience", n = 100)  # Con o sin #

# Buscar menciones a un usuario
mentions <- search_mentions("hadleywickham", n = 50)
```

### 3. Tweets de Usuario (3 funciones)

```r
# Obtener tweets de un usuario
tweets <- user_tweets("rstudio", n = 100)

# Obtener tweets Y respuestas de un usuario
tweets_replies <- user_tweets_and_replies("hadleywickham", n = 100)

# Obtener solo tweets con media (imágenes/videos)
media_tweets <- user_media("NASA", n = 50)
```

### 4. Información de Usuarios (1 función)

```r
# Obtener información detallada de un usuario
user <- get_user("hadleywickham")
print(user$followers_count)
print(user$following_count)
print(user$description)
```

### 5. Análisis de Tweets Específicos (4 funciones)

```r
# Obtener detalles completos de un tweet
tweet_id <- "1234567890123456789"
details <- tweet_details(tweet_id)

# Obtener respuestas a un tweet
replies <- tweet_replies(tweet_id, n = 50)

# Obtener quién retuiteó un tweet
retweeters <- get_retweeters(tweet_id, n = 100)

# Obtener retweeters para varios tweets
tweets <- search_tweets("rstats", n = 10)
retweeters <- get_retweeters_batch(tweets, n = 50)
retweeters_df <- to_dataframe(retweeters)  # incluye source_tweet_id

# Si querés mantenerlos agrupados por tweet
retweeters_by_tweet <- get_retweeters_batch(tweets, n = 50, flatten = FALSE)
```

### 6. Análisis de Redes (3 funciones)

```r
# Obtener seguidores de un usuario
followers <- get_followers("rstudio", n = 100)

# Obtener a quién sigue un usuario
following <- get_following("hadleywickham", n = 100)

# Obtener solo seguidores verificados
verified <- verified_followers("elonmusk", n = 50)
```

### 7. Conversión de Datos (1 función)

```r
# Convertir tweets a dataframe
tweets <- search_tweets("rstats", n = 100)
df_tweets <- to_dataframe(tweets)

# También funciona con usuarios
followers <- get_followers("rstudio", n = 50)
df_users <- to_dataframe(followers)
```

### 8. Filtrado y Ordenamiento (3 funciones)

```r
# Filtrar tweets por idioma
tweets_es <- filter_by_lang(tweets, "es")
tweets_en <- filter_by_lang(tweets, "en")

# Filtrar por fecha
tweets_recent <- filter_by_date(tweets, from = "2025-10-01")
tweets_range <- filter_by_date(
  tweets,
  from = "2025-10-01",
  to = "2025-10-26"
)

# Ordenar tweets
top_tweets <- sort_tweets(tweets, by = "like_count")
recent_tweets <- sort_tweets(tweets, by = "date", decreasing = TRUE)
```

### 9. Exportación (2 funciones)

```r
# Guardar como CSV
save_csv(tweets, "tweets.csv")

# Guardar como JSON
save_json(tweets, "tweets.json")
```


## 📊 Análisis de Datos

### Ejemplo Completo

```r
library(twscrapeR)
library(dplyr)
library(ggplot2)

# 1. Configurar
setup_twscraper()
add_account(...)

# 2. Buscar tweets
tweets <- search_tweets("#rstats", n = 1000)

# 3. Convertir a dataframe
df <- to_dataframe(tweets)

# 4. Análisis con dplyr
df %>%
  filter(lang == "en") %>%
  arrange(desc(like_count)) %>%
  select(username, text, like_count, retweet_count) %>%
  head(10)

# 5. Visualización
df %>%
  count(date = as.Date(date)) %>%
  ggplot(aes(date, n)) +
  geom_line() +
  labs(title = "Tweets sobre #rstats por día",
       x = "Fecha", y = "Número de tweets")

# 6. Top usuarios
df %>%
  count(username, sort = TRUE) %>%
  head(10) %>%
  ggplot(aes(reorder(username, n), n)) +
  geom_col() +
  coord_flip() +
  labs(title = "Top 10 usuarios", x = "", y = "Tweets")
```

## 🔧 Solución de Problemas

### Python no encontrado

Si `setup_twscraper()` no encuentra Python:

```r
# Especificar ruta a Python ya instalado
setup_twscraper(python_path = "C:/Python/python.exe")
```

Instalá Python 3.10+ manualmente desde <https://www.python.org/downloads/> o con tu gestor de entornos preferido. Después instalá `twscrape` en ese entorno y ejecutá `setup_twscraper()` nuevamente.

### Verificar configuración

```r
# Verificar si todo está correcto
check_setup()

# Ver configuración de Python
reticulate::py_config()
```

### Cuenta no activa

Si una cuenta aparece como `active = FALSE`:

```r
# 1. Listar cuentas para ver el estado
list_accounts()

# 2. Eliminar la cuenta inactiva
delete_account("usuario")

# 3. Volver a agregarla CON cookies válidas y actuales
add_account(
  username = "usuario",
  password = "pass",
  email = "email@example.com",
  email_password = "email_pass",
  cookies = "auth_token=...; ct0=..."  # OBLIGATORIO
)
```

**Importante**:
- Las cookies son OBLIGATORIAS para activar cuentas
- Si una cuenta no se activa, las cookies están expiradas o son inválidas
- Obtén nuevas cookies desde tu navegador (ver sección "¿Cómo obtener cookies?")

### Rate Limiting

Twitter/X aplica límites por tipo de consulta y `twscrape` los administra por cola. Si ves un mensaje como este:

```text
No account available for queue "Followers". Next available at 20:25:08
```

significa que todas las cuentas disponibles para esa cola llegaron temporalmente a su límite. No es un error de R: hay que esperar o distribuir mejor la carga.

Qué hacer:

1. **Bajá `n`**: empezá con valores chicos (`n = 50` o `n = 100`) y subí de a poco.
2. **Agregá más cuentas válidas**: `twscrape` rota automáticamente entre cuentas activas.
3. **Verificá tus cuentas**: usá `list_accounts()` y confirmá que aparezcan como `active = TRUE`.
4. **Esperá el reset**: el propio mensaje indica cuándo vuelve a estar disponible la cola.

Ejemplo con varias cuentas:

```r
add_account("usuario1", "pass1", "email1@example.com", "email_pass1", "auth_token=...; ct0=...")
add_account("usuario2", "pass2", "email2@example.com", "email_pass2", "auth_token=...; ct0=...")

list_accounts()
```

No hay un número universal que evite siempre el bloqueo: depende de la cola (`Search`, `Followers`, `UserTweets`, etc.), de la edad/estado de las cuentas y de la carga reciente. La regla práctica es pedir lotes más chicos y dejar que la rotación haga su trabajo.

### Error `IndexError: list index out of range` en twscrape

X/Twitter cambia con frecuencia sus bundles web. En `twscrape 0.17.0` eso puede romper el cálculo interno de `x-client-transaction-id` y bloquear consultas reales aunque la cuenta figure activa.

La solución actual es usar `twscrape >= 0.18.0`, donde el arreglo ya viene incluido upstream. `twscrapeR` no aplica un monkeypatch local para este caso.

```sh
python -m pip install "twscrape>=0.18.0"
```

```r
setup_twscraper()
```

Después de actualizar, una prueba corta como `search_tweets("Milei", n = 5)` debería funcionar si la cuenta y las cookies están activas.

## 📚 Estructura de Datos

### Tweet Object

```r
tweet <- tweets[[1]]
str(tweet)
```

Campos disponibles:
- `id`: ID del tweet
- `date`: Fecha y hora (POSIXct)
- `text`: Texto completo del tweet
- `username`: Usuario que publicó
- `user_displayname`: Nombre mostrado del usuario
- `user_id`: ID del usuario
- `reply_count`: Número de respuestas
- `retweet_count`: Número de retweets
- `like_count`: Número de likes
- `quote_count`: Número de quote tweets
- `views_count`: Número de vistas
- `lang`: Idioma del tweet
- `url`: URL del tweet
- `user_followers`: Seguidores del usuario
- `user_verified`: Usuario verificado (TRUE/FALSE)

### User Object

```r
user <- get_user("username")
str(user)
```

Campos:
- `id`: ID del usuario
- `username`: @usuario
- `displayname`: Nombre mostrado
- `description`: Biografía
- `followers_count`: Seguidores
- `following_count`: Siguiendo
- `tweets_count`: Total de tweets
- `verified`: Cuenta verificada
- `created`: Fecha de creación
- `location`: Ubicación
- `url`: URL del perfil
- `profile_image_url`: URL de la foto de perfil

## 🤝 Contribuciones

Las contribuciones son bienvenidas! Por favor:

1. Fork el repositorio
2. Crea una rama para tu feature (`git checkout -b feature/AmazingFeature`)
3. Commit tus cambios (`git commit -m 'Add some AmazingFeature'`)
4. Push a la rama (`git push origin feature/AmazingFeature`)
5. Abre un Pull Request

## 📄 Licencia

MIT License - ver archivo [LICENSE](LICENSE)

## 🙏 Agradecimientos

**twscrapeR** es posible gracias a dos proyectos excepcionales de Python:

* **[snscrape](https://github.com/JustAnotherArchivist/snscrape)** por [JustAnotherArchivist](https://github.com/JustAnotherArchivist): Arquitectura base de scraping, parsers y modelos de datos.

* **[twscrape](https://github.com/vladkens/twscrape)** por [vladkens](https://github.com/vladkens): Sistema multi-cuenta, rate limiting y generación de X-Client-Transaction-ID.

Y al ecosistema de R:

* **[reticulate](https://rstudio.github.io/reticulate/)**: Integración perfecta entre R y Python

## 📧 Soporte

- **Issues**: https://github.com/agusnieto77/twscrapeR/issues
- **Email**: agustin.nieto77@gmail.com

## ⚠️ Disclaimer

Este paquete es para propósitos educativos y de investigación. Asegúrate de cumplir con los términos de servicio de Twitter/X al usarlo.

---

#' @title Agregar Cuenta de Twitter
#' @description Agrega una cuenta de Twitter con cookies y la activa automaticamente. IMPORTANTE: Las cookies son obligatorias para activar la cuenta correctamente.
#' @param username Nombre de usuario
#' @param password Contrasena
#' @param email Email de la cuenta
#' @param email_password Contrasena del email
#' @param cookies Cookies de sesion (OBLIGATORIO - formato: "auth_token=...; ct0=...")
#' @param db_file Archivo de base de datos (default: "accounts.db")
#' @param verbose Si `TRUE`, emite mensajes informativos con `message()`/`warning()`; puede silenciarse con `suppressMessages()`/`suppressWarnings()`.
#' @return Invisibly returns `TRUE` if the account was added and is active; invisibly returns `FALSE` if it was not added, already existed, or could not be activated.
#' @export
#' @examples
#' \donttest{
#' if (check_setup() && interactive()) {
#'   # add_account(
#'   #   username = "your_username",
#'   #   password = "your_password",
#'   #   email = "you@example.com",
#'   #   email_password = "your_email_password",
#'   #   cookies = "auth_token=...; ct0=..."
#'   # )
#' }
#' }
add_account <- function(username, password, email, email_password, cookies, db_file = "accounts.db", verbose = TRUE) {
  if (missing(cookies) || length(cookies) != 1 || is.na(cookies) || !nzchar(cookies)) {
    stop("cookies is required. Use a string like 'auth_token=...; ct0=...'.", call. = FALSE)
  }

  ensure_configured()

  .account_message("Agregando Cuenta de Twitter", verbose = verbose)
  .account_message(sprintf("Usuario: @%s", username), verbose = verbose)

  cookies_str <- sprintf(", cookies=%s", .python_string_literal(cookies))

  result <- tryCatch({
    py_result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API
import sqlite3

async def add_account():
    db_path = %s
    conn = sqlite3.connect(db_path)
    conn.close()
    api = API(db_path)

    try:
        await api.pool.add_account(
            %s,
            %s,
            %s,
            %s%s
        )
        return {'success': True, 'message': 'Cuenta agregada'}
    except Exception as e:
        return {'success': False, 'message': str(e)}

_add_account_result = asyncio.run(add_account())
    ",
      .python_string_literal(db_file),
      .python_string_literal(username),
      .python_string_literal(password),
      .python_string_literal(email),
      .python_string_literal(email_password),
      cookies_str
    ))

    py_result$`_add_account_result`
  }, error = function(e) {
    list(success = FALSE, message = as.character(e))
  })

  if (result$success) {
    accounts_check <- tryCatch({
      py_check <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def check_account():
    api = API(%s)
    accounts = await api.pool.get_all()
    target = [acc for acc in accounts if acc.username == %s]
    if target:
        return {'active': target[0].active}
    return {'active': False}

_check_result = asyncio.run(check_account())
      ", .python_string_literal(db_file), .python_string_literal(username)))
      py_check$`_check_result`
    }, error = function(e) {
      list(active = FALSE)
    })

    if (accounts_check$active) {
      .account_message(sprintf("Cuenta @%s agregada y activada correctamente", username), verbose = verbose)
      .account_message("Listo. Ahora puedes usar search_tweets(), user_tweets() o search_hashtag().", verbose = verbose)
      return(invisible(TRUE))
    }

    .account_warning("Cuenta agregada pero no esta activa. Puede deberse a cookies invalidas o expiradas, credenciales incorrectas, o una cuenta suspendida/bloqueada.", verbose = verbose)
    .account_message("Solucion: verifica tus cookies en el navegador y vuelve a intentar.", verbose = verbose)
    return(invisible(FALSE))
  }

  if (grepl("already exists", result$message, ignore.case = TRUE)) {
    .account_warning(sprintf("La cuenta @%s ya existe. Usa list_accounts() para ver tus cuentas.", username), verbose = verbose)
  } else {
    .account_warning(sprintf("Error: %s", result$message), verbose = verbose)
  }

  return(invisible(FALSE))
}

#' @title Agregar Cuenta desde Variables de Entorno
#' @description Lee credenciales desde `.Renviron` o variables de entorno y agrega la cuenta sin exponer secretos en el codigo.
#' @param prefix Prefijo de las variables de entorno (default: "TWS_")
#' @param db_file Archivo de base de datos (default: "accounts.db")
#' @param verbose Si `TRUE`, emite mensajes informativos con `message()`/`warning()`; puede silenciarse con `suppressMessages()`/`suppressWarnings()`.
#' @return `TRUE` si la cuenta se agrego y activo; `FALSE` si se agrego pero no quedo activa o hubo error
#' @export
#' @examples
#' \donttest{
#' # En .Renviron:
#' # TWS_USERNAME='mi_usuario'
#' # TWS_PASSWORD='mi_password'
#' # TWS_EMAIL='mi@email.com'
#' # TWS_EMAIL_PASSWORD='email_pass'
#' # TWS_AUTH_TOKEN='auth_token_del_navegador'
#' # TWS_CT0='ct0_del_navegador'
#'
#' if (check_setup() && interactive()) {
#'   add_account_from_env()
#'
#'   # Para una segunda cuenta, usa otro prefijo:
#'   # TWS2_USERNAME='otra_cuenta'
#'   # TWS2_PASSWORD='otra_password'
#'   # ...
#'   add_account_from_env(prefix = "TWS2_")
#' }
#' }
add_account_from_env <- function(prefix = "TWS_", db_file = "accounts.db", verbose = TRUE) {
  account <- .account_env_values(prefix)

  .account_message(sprintf("Leyendo credenciales desde variables de entorno con prefijo %s", prefix), verbose = verbose)

  add_account(
    username = account$username,
    password = account$password,
    email = account$email,
    email_password = account$email_password,
    cookies = account$cookies,
    db_file = db_file,
    verbose = verbose
  )
}

.account_env_values <- function(prefix = "TWS_") {
  required_vars <- paste0(prefix, c("USERNAME", "PASSWORD", "EMAIL", "EMAIL_PASSWORD"))
  values <- .get_required_env(required_vars)

  list(
    username = values[[paste0(prefix, "USERNAME")]],
    password = values[[paste0(prefix, "PASSWORD")]],
    email = values[[paste0(prefix, "EMAIL")]],
    email_password = values[[paste0(prefix, "EMAIL_PASSWORD")]],
    cookies = .cookies_from_env(prefix)
  )
}

.cookies_from_env <- function(prefix = "TWS_") {
  cookies_var <- paste0(prefix, "COOKIES")
  cookies <- Sys.getenv(cookies_var, unset = NA_character_)
  has_cookies <- !is.na(cookies) && nzchar(cookies)

  if (has_cookies) {
    return(cookies)
  }

  auth_token_var <- paste0(prefix, "AUTH_TOKEN")
  ct0_var <- paste0(prefix, "CT0")
  token_values <- Sys.getenv(c(auth_token_var, ct0_var), unset = NA_character_)

  missing_cookie_vars <- names(token_values)[is.na(token_values) | !nzchar(token_values)]

  if (length(missing_cookie_vars) > 0) {
    stop(
      "Faltan cookies. Configura ", cookies_var,
      " o las variables: ", paste(c(auth_token_var, ct0_var), collapse = ", "),
      call. = FALSE
    )
  }

  paste0("auth_token=", token_values[[auth_token_var]], "; ct0=", token_values[[ct0_var]])
}

.get_required_env <- function(vars) {
  values <- Sys.getenv(vars, unset = NA_character_)
  missing_vars <- vars[is.na(values) | !nzchar(values)]

  if (length(missing_vars) > 0) {
    stop(
      "Faltan variables de entorno requeridas: ", paste(missing_vars, collapse = ", "),
      call. = FALSE
    )
  }

  as.list(stats::setNames(values, vars))
}

.python_string_literal <- function(x) {
  if (length(x) != 1 || is.na(x)) {
    stop("Se esperaba un string no vac\u00edo para construir el literal de Python.", call. = FALSE)
  }

  x <- as.character(x)
  x <- gsub("\\\\", "\\\\\\\\", x)
  x <- gsub("'", "\\\\'", x)
  x <- gsub("\n", "\\n", x, fixed = TRUE)
  x <- gsub("\r", "\\r", x, fixed = TRUE)
  paste0("'", x, "'")
}

#' @title Listar Cuentas
#' @description Lista todas las cuentas configuradas
#' @param db_file Archivo de base de datos (default: "accounts.db")
#' @param verbose Si `TRUE`, emite mensajes informativos con `message()`/`warning()`; puede silenciarse con `suppressMessages()`/`suppressWarnings()`.
#' @return Invisibly returns a list of configured accounts. Each account contains `username`, `email`, `active`, and `locks`; an empty list indicates no accounts or an error.
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   accounts <- list_accounts()
#'   print(accounts)
#' }
#' }
list_accounts <- function(db_file = "accounts.db", verbose = TRUE) {
  ensure_configured()

  result <- tryCatch({
    py_result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def list_accounts():
    try:
        api = API(%s)
        accounts = await api.pool.get_all()
        return {
            'success': True,
            'accounts': [{
                'username': acc.username,
                'email': acc.email,
                'active': acc.active,
                'locks': acc.locks
            } for acc in accounts]
        }
    except Exception as e:
        return {'success': False, 'message': str(e), 'accounts': []}

_list_accounts_result = asyncio.run(list_accounts())
  ", .python_string_literal(db_file)))

    py_result$`_list_accounts_result`
  }, error = function(e) {
    error_msg <- as.character(e)
    list(success = FALSE, message = error_msg, accounts = list())
  })

  if (!result$success) {
    if (grepl("no such table", result$message, ignore.case = TRUE)) {
      .account_warning("No hay cuentas configuradas aun. Para comenzar, agrega tu primera cuenta con add_account(...).", verbose = verbose)
    } else {
      .account_warning(sprintf("Error al listar cuentas: %s", result$message), verbose = verbose)
    }
    return(invisible(list()))
  }

  accounts <- result$accounts

  if (length(accounts) == 0) {
    .account_warning("No hay cuentas configuradas. Agrega una cuenta con add_account(...).", verbose = verbose)
    return(invisible(list()))
  }

  .account_message(sprintf("Cuentas configuradas: %d", length(accounts)), verbose = verbose)

  df <- data.frame(
    username = sapply(accounts, function(x) x$username),
    email = sapply(accounts, function(x) x$email),
    active = sapply(accounts, function(x) x$active),
    stringsAsFactors = FALSE
  )

  if (isTRUE(verbose)) {
    message(paste(utils::capture.output(print(df)), collapse = "\n"))
  }

  return(invisible(accounts))
}

#' @title Eliminar Cuenta
#' @description Elimina una cuenta de la base de datos
#' @param username Nombre de usuario a eliminar
#' @param db_file Archivo de base de datos (default: "accounts.db")
#' @param verbose Si `TRUE`, emite mensajes informativos con `message()`/`warning()`; puede silenciarse con `suppressMessages()`/`suppressWarnings()`.
#' @return Invisibly returns `TRUE` if the account was deleted; invisibly returns `FALSE` otherwise.
#' @export
#' @examples
#' \donttest{
#' if (check_setup() && interactive()) {
#'   # delete_account("your_username")
#' }
#' }
delete_account <- function(username, db_file = "accounts.db", verbose = TRUE) {
  ensure_configured()

  .account_message(sprintf("Eliminando cuenta: @%s", username), verbose = verbose)

  result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def delete_account():
    api = API(%s)
    try:
        await api.pool.delete_accounts(%s)
        return {'success': True}
    except Exception as e:
        return {'success': False, 'message': str(e)}

_delete_result = asyncio.run(delete_account())
  ", .python_string_literal(db_file), .python_string_literal(username)))

  res <- result$`_delete_result`

  if (res$success) {
    .account_message("Cuenta eliminada", verbose = verbose)
  } else {
    .account_warning(sprintf("Error: %s", res$message), verbose = verbose)
  }

  return(invisible(res$success))
}

.account_message <- function(text, verbose = TRUE) {
  if (isTRUE(verbose)) {
    message(text)
  }
}

.account_warning <- function(text, verbose = TRUE) {
  if (isTRUE(verbose)) {
    warning(text, call. = FALSE)
  }
}

#' @keywords internal
.onLoad <- function(libname, pkgname) {
  # Suprimir avisos de cli sobre tema desconocido de RStudio
  options(cli.ignore_unknown_rstudio_theme = TRUE)
}

#' @keywords internal
.onAttach <- function(libname, pkgname) {
  # Mensaje de bienvenida
  packageStartupMessage("twscrapeR v", utils::packageVersion(pkgname), " - Twitter/X Scraping via Python")
  packageStartupMessage("Use setup_twscraper() para comenzar")

  # Verificar si ya esta configurado
  if (.twscraper_env$configured) {
    packageStartupMessage("\u2713 twscrapeR configurado y listo")
  }
}

#' @title Convertir Tweets o Usuarios a Data Frame
#' @description Convierte una lista de tweets o usuarios a data.frame usando purrr::map_dfr
#' @param x Lista de tweets o usuarios
#' @return data.frame o tibble
#' @export
#' @examples
#' \donttest{
#' tweets <- list(
#'   list(
#'     id = "1", date = as.POSIXct("2026-01-01"), text = "Hello R",
#'     username = "rstats", user_displayname = "R Stats", user_id = "10",
#'     reply_count = 0, retweet_count = 1, like_count = 2, quote_count = 0,
#'     views_count = 100, lang = "en", url = "https://example.com/1",
#'     user_followers = 1000, user_verified = FALSE
#'   )
#' )
#' class(tweets) <- c("twscraper_tweets", "list")
#' df <- to_dataframe(tweets)
#'
#' users <- list(list(
#'   id = "10", username = "rstats", displayname = "R Stats",
#'   description = "Example user", followers_count = 1000,
#'   following_count = 50, tweets_count = 200, verified = FALSE,
#'   created = "2026-01-01", location = "", url = "",
#'   profile_image_url = ""
#' ))
#' class(users) <- c("twscraper_users", "list")
#' users_df <- to_dataframe(users)
#'
#' users[[1]]$source_tweet_id <- "1"
#' retweeters_df <- to_dataframe(users)  # incluye source_tweet_id
#' }
to_dataframe <- function(x) {
  # Check if it's a single user object (has 12 elements with class twscraper_user)
  if (inherits(x, "twscraper_user")) {
    # Single user object with 12 fields - convert with as_tibble
    if (requireNamespace("tibble", quietly = TRUE)) {
      return(tibble::as_tibble(x))
    } else {
      return(as.data.frame(x, stringsAsFactors = FALSE))
    }
  }

  # Check if it's a single tweet object
  if (inherits(x, "twscraper_tweet")) {
    # Single tweet object - convert with as_tibble
    if (requireNamespace("tibble", quietly = TRUE)) {
      return(tibble::as_tibble(x))
    } else {
      return(as.data.frame(x, stringsAsFactors = FALSE))
    }
  }

  if (length(x) == 0) {
    cli::cli_alert_warning("No hay elementos para convertir")
    if (requireNamespace("tibble", quietly = TRUE)) {
      return(tibble::tibble())
    } else {
      return(data.frame())
    }
  }

  # Check if single element list - use as_tibble directly
  if (length(x) == 1) {
    if (requireNamespace("tibble", quietly = TRUE)) {
      return(tibble::as_tibble(x[[1]]))
    } else {
      return(as.data.frame(x[[1]], stringsAsFactors = FALSE))
    }
  }

  # Use purrr::map_dfr if available with as_tibble, otherwise fallback to base R
  if (requireNamespace("purrr", quietly = TRUE) && requireNamespace("tibble", quietly = TRUE)) {
    df <- purrr::map_dfr(x, ~tibble::as_tibble(.x))
  } else {
    # Fallback to base R - detect if users or tweets
    first_elem <- x[[1]]
    is_users <- !is.null(first_elem$displayname) && is.null(first_elem$text)

    if (is_users) {
      df <- data.frame(
        user_id = sapply(x, function(u) u$id %||% NA_character_),
        username = sapply(x, function(u) u$username %||% NA_character_),
        displayname = sapply(x, function(u) u$displayname %||% NA_character_),
        description = sapply(x, function(u) u$description %||% NA_character_),
        followers_count = sapply(x, function(u) as.numeric(u$followers_count %||% 0)),
        following_count = sapply(x, function(u) as.numeric(u$following_count %||% 0)),
        tweets_count = sapply(x, function(u) as.numeric(u$tweets_count %||% 0)),
        verified = sapply(x, function(u) isTRUE(u$verified)),
        created = sapply(x, function(u) u$created %||% NA_character_),
        location = sapply(x, function(u) u$location %||% NA_character_),
        url = sapply(x, function(u) u$url %||% NA_character_),
        profile_image_url = sapply(x, function(u) u$profile_image_url %||% NA_character_),
        stringsAsFactors = FALSE
      )

      has_source_tweet_id <- any(vapply(x, function(u) !is.null(u$source_tweet_id), logical(1)))
      if (has_source_tweet_id) {
        df <- data.frame(
          source_tweet_id = sapply(x, function(u) u$source_tweet_id %||% NA_character_),
          df,
          stringsAsFactors = FALSE
        )
      }
    } else {
      df <- data.frame(
        tweet_id = sapply(x, function(t) t$id %||% NA_character_),
        date = as.POSIXct(sapply(x, function(t) as.character(t$date %||% NA))),
        username = sapply(x, function(t) t$username %||% NA_character_),
        user_displayname = sapply(x, function(t) t$user_displayname %||% NA_character_),
        user_id = sapply(x, function(t) t$user_id %||% NA_character_),
        text = sapply(x, function(t) t$text %||% NA_character_),
        reply_count = as.numeric(sapply(x, function(t) t$reply_count %||% 0)),
        retweet_count = as.numeric(sapply(x, function(t) t$retweet_count %||% 0)),
        like_count = as.numeric(sapply(x, function(t) t$like_count %||% 0)),
        quote_count = as.numeric(sapply(x, function(t) t$quote_count %||% 0)),
        views_count = as.numeric(sapply(x, function(t) t$views_count %||% 0)),
        lang = sapply(x, function(t) t$lang %||% NA_character_),
        url = sapply(x, function(t) t$url %||% NA_character_),
        user_followers = as.numeric(sapply(x, function(t) t$user_followers %||% 0)),
        user_verified = as.logical(sapply(x, function(t) t$user_verified %||% FALSE)),
        stringsAsFactors = FALSE
      )
    }
  }

  return(df)
}

#' @title Guardar Tweets como CSV
#' @description Guarda tweets en formato CSV
#' @param tweets Lista de tweets
#' @param file Nombre del archivo
#' @return Invisibly returns the output file path. The CSV contains one row per tweet/user-like item after conversion with `to_dataframe()`.
#' @export
#' @examples
#' \donttest{
#' tweets <- list(list(id = "1", text = "Hello R", username = "rstats"))
#' save_csv(tweets, tempfile(fileext = ".csv"))
#' }
save_csv <- function(tweets, file) {
  df <- to_dataframe(tweets)
  utils::write.csv(df, file, row.names = FALSE, fileEncoding = "UTF-8")
  cli::cli_alert_success("Guardado en: {file}")
  cli::cli_text("  {nrow(df)} tweets")
  return(invisible(file))
}

#' @title Guardar Tweets como JSON
#' @description Guarda tweets en formato JSON
#' @param tweets Lista de tweets
#' @param file Nombre del archivo
#' @return Invisibly returns the output file path. The JSON file contains the list structure supplied in `tweets`, written with `jsonlite::write_json()`.
#' @export
#' @examples
#' \donttest{
#' tweets <- list(list(id = "1", text = "Hello R", username = "rstats"))
#' save_json(tweets, tempfile(fileext = ".json"))
#' }
save_json <- function(tweets, file) {
  if (!requireNamespace("jsonlite", quietly = TRUE)) {
    stop("El paquete 'jsonlite' es necesario y debe estar disponible antes de usar save_json().", call. = FALSE)
  }

  jsonlite::write_json(tweets, file, pretty = TRUE, auto_unbox = TRUE)
  cli::cli_alert_success("Guardado en: {file}")
  cli::cli_text("  {length(tweets)} tweets")
  return(invisible(file))
}

#' @title Metodo Print para Tweets
#' @description Imprime resumen de tweets
#' @param x Lista de tweets
#' @param ... Argumentos adicionales
#' @return Invisibly returns `x`, a `twscraper_tweets` list. Each element represents one tweet with fields such as `id`, `date`, `text`, `username`, engagement counts, language, and URL.
#' @export
print.twscraper_tweets <- function(x, ...) {
  cli::cli_h2("Tweets de twscrapeR")
  cli::cli_text("")
  cli::cli_alert_info("Total: {length(x)} tweets")

  if (length(x) > 0) {
    cli::cli_text("")
    cli::cli_text("{.strong Primeros 3 tweets:}")
    cli::cli_text("")

    for (i in 1:min(3, length(x))) {
      tw <- x[[i]]
      cli::cli_text("{i}. @{tw$username} ({tw$user_displayname})")
      cli::cli_text("   {substr(tw$text, 1, 70)}...")
      cli::cli_text("   \u2764 {tw$like_count} | \U0001f501 {tw$retweet_count} | \U0001f441 {tw$views_count}")
      cli::cli_text("")
    }

    if (length(x) > 3) {
      cli::cli_text("   ... y {length(x) - 3} tweets m\u00e1s")
    }

    cli::cli_text("")
    cli::cli_text("{.emph Usa to_dataframe() para convertir a data.frame}")
  }

  invisible(x)
}

#' @title Metodo Print para Usuario
#' @description Imprime informacion de usuario
#' @param x Informacion de usuario
#' @param ... Argumentos adicionales
#' @return Invisibly returns `x`, a `twscraper_user` list with fields such as `id`, `username`, `displayname`, `description`, follower/following counts, verification status, location, URL, and profile image URL.
#' @export
print.twscraper_user <- function(x, ...) {
  cli::cli_h2("Usuario de Twitter")
  cli::cli_text("")

  cli::cli_dl(c(
    "Username" = paste0("@", x$username),
    "Nombre" = x$displayname,
    "ID" = x$id,
    "Descripci\u00f3n" = substr(x$description, 1, 100),
    "Seguidores" = format(x$followers_count, big.mark = ","),
    "Siguiendo" = format(x$following_count, big.mark = ","),
    "Tweets" = format(x$tweets_count, big.mark = ","),
    "Verificado" = if (x$verified) "S\u00ed" else "No",
    "Ubicaci\u00f3n" = x$location,
    "URL" = x$url
  ))

  invisible(x)
}

#' @title Metodo Print para Lista de Usuarios
#' @description Imprime resumen de usuarios
#' @param x Lista de usuarios
#' @param ... Argumentos adicionales
#' @return Invisibly returns `x`, a `twscraper_users` list. Each element represents one user with profile metadata and account statistics.
#' @export
print.twscraper_users <- function(x, ...) {
  cli::cli_h2("Usuarios de twscrapeR")
  cli::cli_text("")
  cli::cli_alert_info("Total: {length(x)} usuarios")

  if (length(x) > 0) {
    cli::cli_text("")
    cli::cli_text("{.strong Primeros 5 usuarios:}")
    cli::cli_text("")

    for (i in 1:min(5, length(x))) {
      user <- x[[i]]
      cli::cli_text("{i}. @{user$username} ({user$displayname})")

      # Descripcion truncada
      desc <- user$description %||% ""
      if (nchar(desc) > 60) {
        desc <- paste0(substr(desc, 1, 60), "...")
      }
      if (nchar(desc) > 0) {
        cli::cli_text("   {desc}")
      }

      cli::cli_text("   \U0001f465 {format(user$followers_count, big.mark = ',')} seguidores | \U0001f4dd {format(user$tweets_count, big.mark = ',')} tweets")
      cli::cli_text("")
    }

    if (length(x) > 5) {
      cli::cli_text("   ... y {length(x) - 5} usuarios m\u00e1s")
    }

    cli::cli_text("")
    cli::cli_text("{.emph Usa to_dataframe() para convertir a data.frame}")
  }

  invisible(x)
}

#' @title Metodo Print para Tweet Individual
#' @description Imprime informacion de un tweet individual
#' @param x Informacion de un tweet
#' @param ... Argumentos adicionales
#' @return Invisibly returns `x`, a `twscraper_tweet` list with tweet metadata, author fields, engagement counts, language, and URL.
#' @export
print.twscraper_tweet <- function(x, ...) {
  cli::cli_h2("Tweet de Twitter")
  cli::cli_text("")

  cli::cli_text("{.strong Autor:} @{x$username} ({x$user_displayname})")
  cli::cli_text("{.strong ID:} {x$id}")
  cli::cli_text("{.strong Fecha:} {format(x$date, '%Y-%m-%d %H:%M:%S')}")
  cli::cli_text("")

  # Mostrar el texto completo
  cli::cli_text("{.strong Texto:}")
  cli::cli_text("  {x$text}")
  cli::cli_text("")

  # Estadisticas
  cli::cli_text("{.strong Estad\u00edsticas:}")
  cli::cli_text("  \u2764  Likes: {format(x$like_count, big.mark = ',')}")
  cli::cli_text("  \U0001f501 Retweets: {format(x$retweet_count, big.mark = ',')}")
  cli::cli_text("  \U0001f4ac Respuestas: {format(x$reply_count, big.mark = ',')}")
  cli::cli_text("  \U0001f516 Quotes: {format(x$quote_count, big.mark = ',')}")
  cli::cli_text("  \U0001f441  Vistas: {format(x$views_count, big.mark = ',')}")
  cli::cli_text("")

  cli::cli_text("{.strong Idioma:} {x$lang}")
  cli::cli_text("{.strong URL:} {x$url}")
  cli::cli_text("")
  cli::cli_text("{.emph Usa to_dataframe() para convertir a data.frame}")

  invisible(x)
}

#' @title Convertir Usuario a Data Frame
#' @description Convierte un usuario individual a data.frame de una fila
#' @param x Usuario de get_user()
#' @param ... Argumentos adicionales
#' @return data.frame
#' @export
as.data.frame.twscraper_user <- function(x, ...) {
  data.frame(
    user_id = x$id %||% NA_character_,
    username = x$username %||% NA_character_,
    displayname = x$displayname %||% NA_character_,
    description = x$description %||% NA_character_,
    followers_count = as.numeric(x$followers_count %||% 0),
    following_count = as.numeric(x$following_count %||% 0),
    tweets_count = as.numeric(x$tweets_count %||% 0),
    verified = isTRUE(x$verified),
    created = x$created %||% NA_character_,
    location = x$location %||% NA_character_,
    url = x$url %||% NA_character_,
    profile_image_url = x$profile_image_url %||% NA_character_,
    stringsAsFactors = FALSE
  )
}

#' @title Filtrar Tweets por Idioma
#' @description Filtra tweets por idioma
#' @param tweets Lista de tweets
#' @param lang Codigo de idioma (ej: "es", "en", "pt")
#' @return Lista filtrada de tweets
#' @export
#' @examples
#' \donttest{
#' tweets <- list(
#'   list(id = "1", lang = "es"),
#'   list(id = "2", lang = "en")
#' )
#' class(tweets) <- c("twscraper_tweets", "list")
#' tweets_es <- filter_by_lang(tweets, "es")
#' }
filter_by_lang <- function(tweets, lang) {
  filtered <- Filter(function(tw) tw$lang == lang, tweets)
  cli::cli_alert_info("Filtrados {length(filtered)} tweets en idioma '{lang}'")
  class(filtered) <- class(tweets)
  return(filtered)
}

#' @title Filtrar Tweets por Fecha
#' @description Filtra tweets por rango de fechas
#' @param tweets Lista de tweets
#' @param from Fecha inicial (POSIXct o character)
#' @param to Fecha final (POSIXct o character)
#' @return Lista filtrada de tweets
#' @export
#' @examples
#' \donttest{
#' tweets <- list(
#'   list(id = "1", date = as.POSIXct("2026-01-01")),
#'   list(id = "2", date = as.POSIXct("2026-02-01"))
#' )
#' class(tweets) <- c("twscraper_tweets", "list")
#' tweets_recent <- filter_by_date(tweets, from = "2025-10-01")
#' }
filter_by_date <- function(tweets, from = NULL, to = NULL) {
  if (!is.null(from)) {
    from <- as.POSIXct(from)
    tweets <- Filter(function(tw) tw$date >= from, tweets)
  }

  if (!is.null(to)) {
    to <- as.POSIXct(to)
    tweets <- Filter(function(tw) tw$date <= to, tweets)
  }

  cli::cli_alert_info("Filtrados {length(tweets)} tweets por fecha")
  class(tweets) <- c("twscraper_tweets", "list")
  return(tweets)
}

#' @title Ordenar Tweets
#' @description Ordena tweets por algun criterio
#' @param tweets Lista de tweets
#' @param by Campo por el cual ordenar ("date", "like_count", "retweet_count", "views_count")
#' @param decreasing Orden descendente (default: TRUE)
#' @return Lista ordenada de tweets
#' @export
#' @examples
#' \donttest{
#' tweets <- list(
#'   list(id = "1", date = as.POSIXct("2026-01-01"), like_count = 2),
#'   list(id = "2", date = as.POSIXct("2026-02-01"), like_count = 10)
#' )
#' class(tweets) <- c("twscraper_tweets", "list")
#' top_tweets <- sort_tweets(tweets, by = "like_count")
#' }
sort_tweets <- function(tweets, by = "date", decreasing = TRUE) {
  sorted <- tweets[order(sapply(tweets, function(x) x[[by]]), decreasing = decreasing)]
  class(sorted) <- class(tweets)
  return(sorted)
}

#' @title Setup twscrapeR
#' @description Checks that Python and the Python `twscrape` package are available and configures the active reticulate session. It does not install Python or Python packages automatically.
#' @param python_path Ruta a Python (opcional, se detecta automaticamente)
#' @param install_python Deprecated compatibility argument. Ignored; installation must be performed by the user outside package functions.
#' @param ask Deprecated compatibility argument. Ignored; retained to avoid breaking existing calls.
#' @return Invisibly returns `TRUE` when Python >= 3.10 and the Python `twscrape` module are available and configured. Invisibly returns `FALSE` otherwise, after printing installation instructions.
#' @export
#' @examples
#' \donttest{
#' if (interactive()) {
#'   setup_twscraper()
#' }
#' }
setup_twscraper <- function(python_path = NULL, install_python = TRUE, ask = TRUE) {
  cli::cli_h1("Configuracion de twscrapeR")
  cli::cli_text("")

  if (!missing(install_python) || !missing(ask)) {
    cli::cli_alert_info("Los argumentos 'install_python' y 'ask' se conservan por compatibilidad, pero ya no instalan software automaticamente.")
  }

  cli::cli_h2("Paso 1: Verificacion de Python")

  python_found <- FALSE

  if (!is.null(python_path)) {
    cli::cli_alert_info("Usando Python especificado: {python_path}")
    if (file.exists(python_path)) {
      reticulate::use_python(python_path, required = TRUE)
      python_found <- TRUE
    } else {
      cli::cli_alert_danger("Python no encontrado en: {python_path}")
    }
  }

  if (!python_found) {
    cli::cli_alert_info("Buscando Python en el sistema...")

    tryCatch({
      reticulate::py_available(initialize = TRUE)
      python_found <- TRUE
      python_path <- reticulate::py_config()$python
      cli::cli_alert_success("Python encontrado: {python_path}")
    }, error = function(e) {
      cli::cli_alert_warning("Python no detectado automaticamente")
    })
  }

  if (python_found) {
    tryCatch({
      py_version <- as.character(reticulate::py_config()$version)
      version_parts <- as.numeric(strsplit(py_version, "\\.")[[1]][1:2])
      python_version_ok <- (version_parts[1] > 3) || (version_parts[1] == 3 && version_parts[2] >= 10)

      if (!python_version_ok) {
        cli::cli_text("")
        cli::cli_alert_warning("Python {py_version} encontrado, pero twscrape requiere Python >= 3.10")
        python_found <- FALSE
      }
    }, error = function(e) {
      cli::cli_alert_warning("No se pudo verificar la version de Python")
      python_found <- FALSE
    })
  }

  if (!python_found) {
    cli::cli_text("")
    cli::cli_alert_warning("Se necesita Python 3.10 o superior")
    cli::cli_text("Instala Python 3.10+ fuera de R y vuelve a ejecutar setup_twscraper().")
    cli::cli_text("Descarga: https://www.python.org/downloads/")
    cli::cli_text("Si usas conda, crea un entorno manualmente, por ejemplo: conda create -n twscraper_env python=3.11")
    return(invisible(FALSE))
  }

  cli::cli_text("")
  cli::cli_h2("Paso 2: Verificacion de twscrape (Python)")

  has_twscrape <- reticulate::py_module_available("twscrape")

  if (!has_twscrape) {
    cli::cli_alert_warning("El modulo Python 'twscrape' no esta instalado en el Python activo")
    cli::cli_text("Instalalo fuera de las funciones del paquete, por ejemplo:")
    cli::cli_code("python -m pip install 'twscrape>=0.18.0'")
    cli::cli_text("Luego vuelve a ejecutar setup_twscraper().")
    return(invisible(FALSE))
  }

  cli::cli_alert_success("twscrape ya esta instalado")

  cli::cli_text("")
  cli::cli_h2("Paso 3: Configuracion final")

  .twscraper_env$python_path <- python_path
  .twscraper_env$twscrape <- reticulate::import("twscrape")
  .twscraper_env$configured <- TRUE

  cli::cli_alert_success("Configuracion completada")
  cli::cli_text("")

  cli::cli_rule("Resumen de Configuracion")
  cli::cli_text("")
  cli::cli_dl(c(
    "Python" = python_path,
    "Version Python" = as.character(reticulate::py_config()$version),
    "twscrape" = "Disponible y listo"
  ))
  cli::cli_text("")

  cli::cli_alert_success("twscrapeR esta listo para usar!")
  cli::cli_text("")

  cli::cli_text("{.strong Proximos pasos:}")
  cli::cli_ol(c(
    "Agregar cuenta: add_account_from_env() o add_account('usuario', 'password', 'email', 'email_pass', cookies='...')",
    "Buscar tweets: search_tweets('query', n = 100)",
    "Ver ayuda: ?twscrapeR"
  ))
  cli::cli_text("")

  return(invisible(TRUE))
}

#' @title Verificar si twscrapeR esta configurado
#' @description Verifica si Python y twscrape estan configurados en la sesion actual.
#' @param ... Ignorado; reservado para compatibilidad futura.
#' @return `TRUE` si twscrapeR esta configurado en la sesion actual; `FALSE` en caso contrario.
#' @export
#' @examples
#' check_setup()
check_setup <- function(...) {
  if (!exists(".twscraper_env") || !.twscraper_env$configured) {
    cli::cli_alert_warning("twscrapeR no esta configurado")
    cli::cli_text("Ejecuta: setup_twscraper()")
    return(FALSE)
  }

  cli::cli_alert_success("twscrapeR esta configurado correctamente")
  cli::cli_dl(c(
    "Python" = .twscraper_env$python_path,
    "twscrape" = "Activo"
  ))
  return(TRUE)
}

#' Entorno global para almacenar configuracion
#' @keywords internal
.twscraper_env <- new.env(parent = emptyenv())
.twscraper_env$configured <- FALSE
.twscraper_env$python_path <- NULL
.twscraper_env$twscrape <- NULL

#' @title Asegurar que twscrapeR esta configurado
#' @description Uso interno - verifica configuracion antes de operaciones
#' @keywords internal
ensure_configured <- function() {
  if (!.twscraper_env$configured) {
    cli::cli_alert_warning("twscrapeR no esta configurado")
    cli::cli_text("Ejecuta setup_twscraper() despues de instalar Python 3.10+ y twscrape>=0.18.0.")
    stop("twscrapeR no esta configurado. Ejecuta setup_twscraper() manualmente.", call. = FALSE)
  }
}

# Funciones Avanzadas de twscrapeR
# Implementacion de funciones adicionales de twscrape

#' @title Obtener Detalles de un Tweet
#' @description Obtiene informacion detallada de un tweet especifico por su ID
#' @param tweet_id ID del tweet (numero o string)
#' @param progress Mostrar progreso (default: TRUE)
#' @return Lista con informacion del tweet o NULL si no se encuentra
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   tweet <- tweet_details(1234567890)
#' }
#' }
tweet_details <- function(tweet_id, progress = TRUE) {
  ensure_configured()

  if (progress) {
    cli::cli_alert_info("Obteniendo detalles del tweet {tweet_id}")
  }

  result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def get_tweet_details():
    api = API()
    tweet = await api.tweet_details(%s)
    if tweet is None:
        return None
    return {
        'id': str(tweet.id),
        'date': str(tweet.date),
        'text': tweet.rawContent,
        'username': tweet.user.username,
        'user_displayname': tweet.user.displayname,
        'user_id': str(tweet.user.id),
        'reply_count': tweet.replyCount or 0,
        'retweet_count': tweet.retweetCount or 0,
        'like_count': tweet.likeCount or 0,
        'quote_count': tweet.quoteCount or 0,
        'views_count': tweet.viewCount or 0,
        'lang': tweet.lang or 'und',
        'url': tweet.url,
        'user_followers': getattr(tweet.user, 'followersCount', 0) or 0,
        'user_verified': getattr(tweet.user, 'verified', False) or False
    }

_tweet_details_result = asyncio.run(get_tweet_details())
  ", tweet_id))

  tweet_info <- result$`_tweet_details_result`

  if (is.null(tweet_info)) {
    if (progress) cli::cli_alert_warning("Tweet no encontrado")
    return(NULL)
  }

  if (progress) {
    cli::cli_alert_success("Tweet obtenido")
  }

  # Convertir fecha a POSIXct
  tweet_info$date <- as.POSIXct(tweet_info$date)

  class(tweet_info) <- c("twscraper_tweet", "list")
  return(tweet_info)
}

#' @title Obtener Respuestas a un Tweet
#' @description Obtiene las respuestas (replies) a un tweet especifico
#' @param tweet_id ID del tweet
#' @param n Numero maximo de respuestas (default: 100)
#' @param progress Mostrar progreso (default: TRUE)
#' @return Lista de tweets (respuestas)
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   replies <- tweet_replies(1234567890, n = 50)
#' }
#' }
tweet_replies <- function(tweet_id, n = 100, progress = TRUE) {
  ensure_configured()

  if (progress) {
    cli::cli_alert_info("Obteniendo respuestas al tweet {tweet_id}")
  }

  result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def get_replies():
    api = API()
    replies = []
    async for tweet in api.tweet_replies(%s, limit=%d):
        replies.append({
            'id': str(tweet.id),
            'date': str(tweet.date),
            'text': tweet.rawContent,
            'username': tweet.user.username,
            'user_displayname': tweet.user.displayname,
            'user_id': str(tweet.user.id),
            'reply_count': tweet.replyCount or 0,
            'retweet_count': tweet.retweetCount or 0,
            'like_count': tweet.likeCount or 0,
            'quote_count': tweet.quoteCount or 0,
            'views_count': tweet.viewCount or 0,
            'lang': tweet.lang or 'und',
            'url': tweet.url,
            'user_followers': getattr(tweet.user, 'followersCount', 0) or 0,
            'user_verified': getattr(tweet.user, 'verified', False) or False
        })
    return replies

_replies_result = asyncio.run(get_replies())
  ", tweet_id, n))

  replies_list <- result$`_replies_result`

  if (progress) {
    cli::cli_alert_success("Encontradas {length(replies_list)} respuestas")
  }

  # Convertir a objetos Tweet
  tweets_list <- lapply(replies_list, function(tw) {
    structure(list(
      id = tw$id,
      date = as.POSIXct(tw$date),
      text = tw$text,
      username = tw$username,
      user_displayname = tw$user_displayname,
      user_id = tw$user_id,
      reply_count = tw$reply_count,
      retweet_count = tw$retweet_count,
      like_count = tw$like_count,
      quote_count = tw$quote_count,
      views_count = tw$views_count,
      lang = tw$lang,
      url = tw$url,
      user_followers = tw$user_followers,
      user_verified = tw$user_verified
    ), class = c("twscraper_tweet", "list"))
  })

  class(tweets_list) <- c("twscraper_tweets", "list")
  return(tweets_list)
}

#' @title Obtener Seguidores de un Usuario
#' @description Obtiene la lista de seguidores de un usuario
#' @param username Nombre de usuario (sin @)
#' @param n Numero maximo de seguidores (default: 100)
#' @param progress Mostrar progreso (default: TRUE)
#' @return Lista de usuarios
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   followers <- get_followers("elonmusk", n = 100)
#' }
#' }
get_followers <- function(username, n = 100, progress = TRUE) {
  ensure_configured()

  if (progress) {
    cli::cli_alert_info("Obteniendo seguidores de @{username}")
  }

  result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def get_user_followers():
    api = API()

    # Primero obtener el user_id
    user = await api.user_by_login('%s')
    user_id = user.id

    followers = []
    async for user in api.followers(user_id, limit=%d):
        followers.append({
            'id': str(user.id),
            'username': getattr(user, 'username', ''),
            'displayname': getattr(user, 'displayname', ''),
            'description': getattr(user, 'rawDescription', '') or getattr(user, 'description', ''),
            'followers_count': getattr(user, 'followersCount', 0) or 0,
            'following_count': getattr(user, 'friendsCount', 0) or 0,
            'tweets_count': getattr(user, 'statusesCount', 0) or 0,
            'verified': getattr(user, 'verified', False) or False,
            'created': str(getattr(user, 'created', '')),
            'location': getattr(user, 'location', '') or '',
            'url': getattr(user, 'url', '') or '',
            'profile_image_url': getattr(user, 'profileImageUrl', '') or ''
        })
    return followers

_followers_result = asyncio.run(get_user_followers())
  ", username, n))

  followers_list <- result$`_followers_result`

  if (progress) {
    cli::cli_alert_success("Encontrados {length(followers_list)} seguidores")
  }

  class(followers_list) <- c("twscraper_users", "list")
  return(followers_list)
}

#' @title Obtener Usuarios Seguidos
#' @description Obtiene la lista de usuarios que sigue un usuario
#' @param username Nombre de usuario (sin @)
#' @param n Numero maximo de usuarios (default: 100)
#' @param progress Mostrar progreso (default: TRUE)
#' @return Lista de usuarios
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   following <- get_following("elonmusk", n = 100)
#' }
#' }
get_following <- function(username, n = 100, progress = TRUE) {
  ensure_configured()

  if (progress) {
    cli::cli_alert_info("Obteniendo usuarios seguidos por @{username}")
  }

  result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def get_user_following():
    api = API()

    # Primero obtener el user_id
    user = await api.user_by_login('%s')
    user_id = user.id

    following = []
    async for user in api.following(user_id, limit=%d):
        following.append({
            'id': str(user.id),
            'username': getattr(user, 'username', ''),
            'displayname': getattr(user, 'displayname', ''),
            'description': getattr(user, 'rawDescription', '') or getattr(user, 'description', ''),
            'followers_count': getattr(user, 'followersCount', 0) or 0,
            'following_count': getattr(user, 'friendsCount', 0) or 0,
            'tweets_count': getattr(user, 'statusesCount', 0) or 0,
            'verified': getattr(user, 'verified', False) or False,
            'created': str(getattr(user, 'created', '')),
            'location': getattr(user, 'location', '') or '',
            'url': getattr(user, 'url', '') or '',
            'profile_image_url': getattr(user, 'profileImageUrl', '') or ''
        })
    return following

_following_result = asyncio.run(get_user_following())
  ", username, n))

  following_list <- result$`_following_result`

  if (progress) {
    cli::cli_alert_success("Encontrados {length(following_list)} usuarios")
  }

  class(following_list) <- c("twscraper_users", "list")
  return(following_list)
}

#' @title Obtener Usuarios que Retuitearon
#' @description Obtiene la lista de usuarios que retuitearon un tweet especifico
#' @param tweet_id ID del tweet
#' @param n Numero maximo de usuarios (default: 100)
#' @param progress Mostrar progreso (default: TRUE)
#' @return Lista de usuarios
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   retweeters <- get_retweeters(1234567890, n = 100)
#' }
#' }
get_retweeters <- function(tweet_id, n = 100, progress = TRUE) {
  ensure_configured()

  if (progress) {
    cli::cli_alert_info("Obteniendo usuarios que retuitearon {tweet_id}")
  }

  result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def get_tweet_retweeters():
    api = API()

    retweeters = []
    async for user in api.retweeters(%s, limit=%d):
        retweeters.append({
            'id': str(user.id),
            'username': getattr(user, 'username', ''),
            'displayname': getattr(user, 'displayname', ''),
            'description': getattr(user, 'rawDescription', '') or getattr(user, 'description', ''),
            'followers_count': getattr(user, 'followersCount', 0) or 0,
            'following_count': getattr(user, 'friendsCount', 0) or 0,
            'tweets_count': getattr(user, 'statusesCount', 0) or 0,
            'verified': getattr(user, 'verified', False) or False,
            'created': str(getattr(user, 'created', '')),
            'location': getattr(user, 'location', '') or '',
            'url': getattr(user, 'url', '') or '',
            'profile_image_url': getattr(user, 'profileImageUrl', '') or ''
        })
    return retweeters

_retweeters_result = asyncio.run(get_tweet_retweeters())
  ", tweet_id, n))

  retweeters_list <- result$`_retweeters_result`

  if (progress) {
    cli::cli_alert_success("Encontrados {length(retweeters_list)} retweeters")
  }

  class(retweeters_list) <- c("twscraper_users", "list")
  return(retweeters_list)
}

#' @title Obtener Retweeters para Varios Tweets
#' @description Obtiene usuarios que retuitearon varios tweets en una sola llamada de R.
#' @param tweets Vector de IDs, lista de tweets `twscraper_tweets`, tweet individual o data.frame con columna `id`/`tweet_id`.
#' @param n Numero maximo de usuarios por tweet (default: 100)
#' @param progress Mostrar progreso (default: TRUE)
#' @param flatten Si es TRUE, devuelve una lista plana de usuarios con columna `source_tweet_id` para usar con `to_dataframe()`. Si es FALSE, devuelve una lista agrupada por tweet.
#' @return Lista de usuarios con `source_tweet_id` o lista agrupada por tweet
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   tweets <- search_tweets("rstats", n = 10)
#'   retweeters <- get_retweeters_batch(tweets, n = 50)
#'   retweeters_df <- to_dataframe(retweeters)
#'
#'   retweeters_by_tweet <- get_retweeters_batch(tweets, n = 50, flatten = FALSE)
#' }
#' }
get_retweeters_batch <- function(tweets, n = 100, progress = TRUE, flatten = TRUE) {
  ensure_configured()

  tweet_ids <- .extract_tweet_ids(tweets)
  tweet_ids <- tweet_ids[!is.na(tweet_ids) & nzchar(tweet_ids)]

  if (length(tweet_ids) == 0) {
    stop("No se encontraron IDs de tweets. Usa un vector de IDs, tweets de twscrapeR o un data.frame con columna 'id'/'tweet_id'.")
  }

  tweet_ids <- unique(tweet_ids)

  if (progress) {
    cli::cli_alert_info("Obteniendo retweeters para {length(tweet_ids)} tweets")
  }

  retweeters_by_tweet <- stats::setNames(vector("list", length(tweet_ids)), tweet_ids)

  for (i in seq_along(tweet_ids)) {
    tweet_id <- tweet_ids[[i]]

    if (progress) {
      cli::cli_alert_info("Procesando tweet {i}/{length(tweet_ids)}: {tweet_id}")
    }

    users <- get_retweeters(tweet_id, n = n, progress = FALSE)
    users <- lapply(users, function(user) {
      user$source_tweet_id <- tweet_id
      user
    })
    class(users) <- c("twscraper_users", "list")

    retweeters_by_tweet[[tweet_id]] <- users
  }

  if (flatten) {
    retweeters <- unlist(retweeters_by_tweet, recursive = FALSE, use.names = FALSE)
    class(retweeters) <- c("twscraper_users", "list")

    if (progress) {
      cli::cli_alert_success("Encontrados {length(retweeters)} retweeters en total")
    }

    return(retweeters)
  }

  class(retweeters_by_tweet) <- c("twscraper_retweeters_batch", "list")

  if (progress) {
    total <- sum(vapply(retweeters_by_tweet, length, integer(1)))
    cli::cli_alert_success("Encontrados {total} retweeters en total")
  }

  return(retweeters_by_tweet)
}

.extract_tweet_ids <- function(tweets) {
  if (is.null(tweets)) {
    return(character())
  }

  if (inherits(tweets, "twscraper_tweet")) {
    return(as.character(tweets$id))
  }

  if (is.data.frame(tweets)) {
    if ("tweet_id" %in% names(tweets)) {
      return(as.character(tweets$tweet_id))
    }

    if ("id" %in% names(tweets)) {
      return(as.character(tweets$id))
    }

    stop("El data.frame debe tener una columna 'tweet_id' o 'id'.")
  }

  if (is.atomic(tweets)) {
    return(as.character(tweets))
  }

  if (is.list(tweets)) {
    ids <- vapply(tweets, function(tweet) {
      if (is.list(tweet) && !is.null(tweet$id)) {
        return(as.character(tweet$id))
      }

      if (is.atomic(tweet) && length(tweet) == 1) {
        return(as.character(tweet))
      }

      NA_character_
    }, character(1))

    return(ids[!is.na(ids)])
  }

  stop("Formato de tweets no soportado. Usa IDs, objetos twscrapeR o un data.frame con 'id'/'tweet_id'.")
}

#' @title Obtener Tweets y Respuestas de un Usuario
#' @description Obtiene tweets y respuestas de un usuario (timeline completo)
#' @param username Nombre de usuario (sin @)
#' @param n Numero maximo de tweets (default: 100)
#' @param progress Mostrar progreso (default: TRUE)
#' @return Lista de tweets
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   all_tweets <- user_tweets_and_replies("elonmusk", n = 100)
#' }
#' }
user_tweets_and_replies <- function(username, n = 100, progress = TRUE) {
  ensure_configured()

  if (progress) {
    cli::cli_alert_info("Obteniendo tweets y respuestas de @{username}")
  }

  result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def get_tweets_and_replies():
    api = API()

    # Primero obtener el user_id
    user = await api.user_by_login('%s')
    user_id = user.id

    tweets = []
    async for tweet in api.user_tweets_and_replies(user_id, limit=%d):
        tweets.append({
            'id': str(tweet.id),
            'date': str(tweet.date),
            'text': tweet.rawContent,
            'username': tweet.user.username,
            'user_displayname': tweet.user.displayname,
            'user_id': str(tweet.user.id),
            'reply_count': tweet.replyCount or 0,
            'retweet_count': tweet.retweetCount or 0,
            'like_count': tweet.likeCount or 0,
            'quote_count': tweet.quoteCount or 0,
            'views_count': tweet.viewCount or 0,
            'lang': tweet.lang or 'und',
            'url': tweet.url,
            'user_followers': getattr(tweet.user, 'followersCount', 0) or 0,
            'user_verified': getattr(tweet.user, 'verified', False) or False
        })
    return tweets

_tweets_and_replies_result = asyncio.run(get_tweets_and_replies())
  ", username, n))

  tweets_list <- result$`_tweets_and_replies_result`

  if (progress) {
    cli::cli_alert_success("Encontrados {length(tweets_list)} tweets")
  }

  # Convertir a objetos Tweet
  tweets_list <- lapply(tweets_list, function(tw) {
    structure(list(
      id = tw$id,
      date = as.POSIXct(tw$date),
      text = tw$text,
      username = tw$username,
      user_displayname = tw$user_displayname,
      user_id = tw$user_id,
      reply_count = tw$reply_count,
      retweet_count = tw$retweet_count,
      like_count = tw$like_count,
      quote_count = tw$quote_count,
      views_count = tw$views_count,
      lang = tw$lang,
      url = tw$url,
      user_followers = tw$user_followers,
      user_verified = tw$user_verified
    ), class = c("twscraper_tweet", "list"))
  })

  class(tweets_list) <- c("twscraper_tweets", "list")
  return(tweets_list)
}

#' @title Obtener Tweets con Media de un Usuario
#' @description Obtiene solo los tweets que contienen imagenes o videos
#' @param username Nombre de usuario (sin @)
#' @param n Numero maximo de tweets (default: 100)
#' @param progress Mostrar progreso (default: TRUE)
#' @return Lista de tweets
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   media_tweets <- user_media("elonmusk", n = 100)
#' }
#' }
user_media <- function(username, n = 100, progress = TRUE) {
  ensure_configured()

  if (progress) {
    cli::cli_alert_info("Obteniendo tweets con media de @{username}")
  }

  result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def get_user_media_tweets():
    api = API()

    # Primero obtener el user_id
    user = await api.user_by_login('%s')
    user_id = user.id

    tweets = []
    async for tweet in api.user_media(user_id, limit=%d):
        tweets.append({
            'id': str(tweet.id),
            'date': str(tweet.date),
            'text': tweet.rawContent,
            'username': tweet.user.username,
            'user_displayname': tweet.user.displayname,
            'user_id': str(tweet.user.id),
            'reply_count': tweet.replyCount or 0,
            'retweet_count': tweet.retweetCount or 0,
            'like_count': tweet.likeCount or 0,
            'quote_count': tweet.quoteCount or 0,
            'views_count': tweet.viewCount or 0,
            'lang': tweet.lang or 'und',
            'url': tweet.url,
            'user_followers': getattr(tweet.user, 'followersCount', 0) or 0,
            'user_verified': getattr(tweet.user, 'verified', False) or False
        })
    return tweets

_media_tweets_result = asyncio.run(get_user_media_tweets())
  ", username, n))

  tweets_list <- result$`_media_tweets_result`

  if (progress) {
    cli::cli_alert_success("Encontrados {length(tweets_list)} tweets con media")
  }

  # Convertir a objetos Tweet
  tweets_list <- lapply(tweets_list, function(tw) {
    structure(list(
      id = tw$id,
      date = as.POSIXct(tw$date),
      text = tw$text,
      username = tw$username,
      user_displayname = tw$user_displayname,
      user_id = tw$user_id,
      reply_count = tw$reply_count,
      retweet_count = tw$retweet_count,
      like_count = tw$like_count,
      quote_count = tw$quote_count,
      views_count = tw$views_count,
      lang = tw$lang,
      url = tw$url,
      user_followers = tw$user_followers,
      user_verified = tw$user_verified
    ), class = c("twscraper_tweet", "list"))
  })

  class(tweets_list) <- c("twscraper_tweets", "list")
  return(tweets_list)
}

#' @title Obtener Seguidores Verificados
#' @description Obtiene solo los seguidores que tienen cuenta verificada
#' @param username Nombre de usuario (sin @)
#' @param n Numero maximo de seguidores (default: 100)
#' @param progress Mostrar progreso (default: TRUE)
#' @return Lista de usuarios verificados
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   verified <- verified_followers("elonmusk", n = 100)
#' }
#' }
verified_followers <- function(username, n = 100, progress = TRUE) {
  ensure_configured()

  if (progress) {
    cli::cli_alert_info("Obteniendo seguidores verificados de @{username}")
  }

  result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def get_verified_followers():
    api = API()

    # Primero obtener el user_id
    user = await api.user_by_login('%s')
    user_id = user.id

    followers = []
    async for user in api.verified_followers(user_id, limit=%d):
        followers.append({
            'id': str(user.id),
            'username': getattr(user, 'username', ''),
            'displayname': getattr(user, 'displayname', ''),
            'description': getattr(user, 'rawDescription', '') or getattr(user, 'description', ''),
            'followers_count': getattr(user, 'followersCount', 0) or 0,
            'following_count': getattr(user, 'friendsCount', 0) or 0,
            'tweets_count': getattr(user, 'statusesCount', 0) or 0,
            'verified': getattr(user, 'verified', False) or False,
            'created': str(getattr(user, 'created', '')),
            'location': getattr(user, 'location', '') or '',
            'url': getattr(user, 'url', '') or '',
            'profile_image_url': getattr(user, 'profileImageUrl', '') or ''
        })
    return followers

_verified_followers_result = asyncio.run(get_verified_followers())
  ", username, n))

  followers_list <- result$`_verified_followers_result`

  if (progress) {
    cli::cli_alert_success("Encontrados {length(followers_list)} seguidores verificados")
  }

  class(followers_list) <- c("twscraper_users", "list")
  return(followers_list)
}


#' @title Convertir Usuarios a Dataframe (alias de to_dataframe)
#' @description Alias interno de to_dataframe() - usar to_dataframe() directamente
#' @param users Lista de usuarios
#' @return data.frame con informacion de usuarios
#' @keywords internal
users_to_dataframe <- function(users) {
  to_dataframe(users)
}

# Operador helper para valores NULL
`%||%` <- function(x, y) if (is.null(x)) y else x

#' @title Buscar Tweets
#' @description Busca tweets usando una consulta
#' @param query Consulta de busqueda (ej: "rstats", "#datascience", "from:usuario")
#' @param n Numero maximo de tweets a retornar (default: 100)
#' @param progress Mostrar barra de progreso (default: TRUE)
#' @param product Tipo de busqueda: "Latest" para tweets recientes o "Top" para tweets destacados (default: "Latest")
#' @return Lista de tweets
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   tweets <- search_tweets("rstats", n = 50)
#'   top_tweets <- search_tweets("rstats", n = 50, product = "Top")
#'   df <- to_dataframe(tweets)
#' }
#' }
search_tweets <- function(query, n = 100, progress = TRUE, product = c("Latest", "Top")) {
  product <- match.arg(product)
  ensure_configured()

  if (progress) {
    cli::cli_alert_info("Buscando tweets: '{query}'")
    cli::cli_progress_step("Obteniendo tweets...")
  }

  result <- reticulate::py_run_string(.search_tweets_python_code(query, n, product))

  tweets <- result$`_search_result`

  if (progress) {
    cli::cli_progress_done()
    cli::cli_alert_success("Encontrados {length(tweets)} tweets")
  }

  # Convertir a estructura R
  tweets_list <- lapply(tweets, function(tw) {
    list(
      id = tw$id,
      date = as.POSIXct(tw$date),
      text = tw$text,
      username = tw$username,
      user_displayname = tw$user_displayname,
      user_id = tw$user_id,
      reply_count = tw$reply_count,
      retweet_count = tw$retweet_count,
      like_count = tw$like_count,
      quote_count = tw$quote_count,
      views_count = tw$views_count,
      lang = tw$lang,
      url = tw$url,
      user_followers = tw$user_followers,
      user_verified = tw$user_verified
    )
  })

  class(tweets_list) <- c("twscraper_tweets", "list")
  return(tweets_list)
}

.search_tweets_python_code <- function(query, n, product = "Latest") {
  query_literal <- .python_string_literal(query)
  product_literal <- .python_string_literal(product)

  sprintf("
import asyncio
from twscrape import API

async def search():
    api = API()
    tweets = []
    async for tweet in api.search(%s, limit=%d, kv={'product': %s}):
        tweets.append({
            'id': str(tweet.id),
            'date': str(tweet.date),
            'text': tweet.rawContent,
            'username': tweet.user.username,
            'user_displayname': tweet.user.displayname,
            'user_id': str(tweet.user.id),
            'reply_count': tweet.replyCount or 0,
            'retweet_count': tweet.retweetCount or 0,
            'like_count': tweet.likeCount or 0,
            'quote_count': tweet.quoteCount or 0,
            'views_count': tweet.viewCount or 0,
            'lang': tweet.lang or 'und',
            'url': tweet.url,
            'user_followers': getattr(tweet.user, 'followersCount', 0) or 0,
            'user_verified': getattr(tweet.user, 'verified', False) or False
        })
    return tweets

_search_result = asyncio.run(search())
  ", query_literal, n, product_literal)
}

#' @title Obtener Tweets de un Usuario
#' @description Obtiene los tweets recientes de un usuario especifico
#' @param username Nombre de usuario (sin @)
#' @param n Numero maximo de tweets (default: 100)
#' @param progress Mostrar progreso (default: TRUE)
#' @return Lista de tweets
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   tweets <- user_tweets("hadleywickham", n = 50)
#' }
#' }
user_tweets <- function(username, n = 100, progress = TRUE) {
  ensure_configured()

  if (progress) {
    cli::cli_alert_info("Obteniendo tweets de @{username}")
  }

  result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def get_user_tweets():
    api = API()

    # Primero obtener el user_id a partir del username
    user = await api.user_by_login('%s')
    user_id = user.id

    tweets = []
    async for tweet in api.user_tweets(user_id, limit=%d):
        tweets.append({
            'id': str(tweet.id),
            'date': str(tweet.date),
            'text': tweet.rawContent,
            'username': tweet.user.username,
            'user_displayname': tweet.user.displayname,
            'user_id': str(tweet.user.id),
            'reply_count': tweet.replyCount or 0,
            'retweet_count': tweet.retweetCount or 0,
            'like_count': tweet.likeCount or 0,
            'quote_count': tweet.quoteCount or 0,
            'views_count': tweet.viewCount or 0,
            'lang': tweet.lang or 'und',
            'url': tweet.url,
            'user_followers': getattr(tweet.user, 'followersCount', 0) or 0,
            'user_verified': getattr(tweet.user, 'verified', False) or False
        })
    return tweets

_user_tweets_result = asyncio.run(get_user_tweets())
  ", username, n))

  tweets <- result$`_user_tweets_result`

  if (progress) {
    cli::cli_alert_success("Encontrados {length(tweets)} tweets de @{username}")
  }

  tweets_list <- lapply(tweets, function(tw) {
    list(
      id = tw$id,
      date = as.POSIXct(tw$date),
      text = tw$text,
      username = tw$username,
      user_displayname = tw$user_displayname,
      user_id = tw$user_id,
      reply_count = tw$reply_count,
      retweet_count = tw$retweet_count,
      like_count = tw$like_count,
      quote_count = tw$quote_count,
      views_count = tw$views_count,
      lang = tw$lang,
      url = tw$url,
      user_followers = tw$user_followers,
      user_verified = tw$user_verified
    )
  })

  class(tweets_list) <- c("twscraper_tweets", "list")
  return(tweets_list)
}

#' @title Obtener Informacion de Usuario
#' @description Obtiene informacion detallada de un usuario
#' @param username Nombre de usuario (sin @)
#' @param progress Mostrar progreso (default: TRUE)
#' @return Lista con informacion del usuario
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   user <- get_user("hadleywickham")
#' }
#' }
get_user <- function(username, progress = TRUE) {
  ensure_configured()

  if (progress) {
    cli::cli_alert_info("Obteniendo informaci\u00f3n de @{username}")
  }

  result <- reticulate::py_run_string(sprintf("
import asyncio
from twscrape import API

async def get_user_info():
    api = API()
    user = await api.user_by_login('%s')
    return {
        'id': str(user.id),
        'username': getattr(user, 'username', ''),
        'displayname': getattr(user, 'displayname', ''),
        'description': getattr(user, 'rawDescription', '') or getattr(user, 'description', ''),
        'followers_count': getattr(user, 'followersCount', 0) or 0,
        'following_count': getattr(user, 'friendsCount', 0) or 0,
        'tweets_count': getattr(user, 'statusesCount', 0) or 0,
        'verified': getattr(user, 'verified', False) or False,
        'created': str(getattr(user, 'created', '')),
        'location': getattr(user, 'location', '') or '',
        'url': getattr(user, 'url', '') or '',
        'profile_image_url': getattr(user, 'profileImageUrl', '') or ''
    }

_user_info_result = asyncio.run(get_user_info())
  ", username))

  user_info <- result$`_user_info_result`

  if (progress) {
    cli::cli_alert_success("Informaci\u00f3n obtenida de @{username}")
  }

  user_data <- list(
    id = user_info$id,
    username = user_info$username,
    displayname = user_info$displayname,
    description = user_info$description,
    followers_count = user_info$followers_count,
    following_count = user_info$following_count,
    tweets_count = user_info$tweets_count,
    verified = user_info$verified,
    created = user_info$created,
    location = user_info$location,
    url = user_info$url,
    profile_image_url = user_info$profile_image_url
  )

  class(user_data) <- c("twscraper_user", "list")
  return(user_data)
}

#' @title Buscar por Hashtag
#' @description Busca tweets que contengan un hashtag especifico
#' @param hashtag Hashtag (con o sin #)
#' @param n Numero maximo de tweets (default: 100)
#' @param progress Mostrar progreso (default: TRUE)
#' @return Lista de tweets
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   tweets <- search_hashtag("rstats", n = 50)
#'   tweets <- search_hashtag("#datascience", n = 100)
#' }
#' }
search_hashtag <- function(hashtag, n = 100, progress = TRUE) {
  # Limpiar hashtag
  hashtag <- gsub("^#", "", hashtag)
  query <- paste0("#", hashtag)

  return(search_tweets(query, n = n, progress = progress))
}

#' @title Buscar Menciones de un Usuario
#' @description Busca tweets que mencionen a un usuario especifico
#' @param username Nombre de usuario (sin @)
#' @param n Numero maximo de tweets (default: 100)
#' @param progress Mostrar progreso (default: TRUE)
#' @return Lista de tweets
#' @export
#' @examples
#' \donttest{
#' if (check_setup()) {
#'   mentions <- search_mentions("hadleywickham", n = 50)
#' }
#' }
search_mentions <- function(username, n = 100, progress = TRUE) {
  username <- gsub("^@", "", username)
  query <- paste0("@", username)

  return(search_tweets(query, n = n, progress = progress))
}

twscrapeR_test_fn <- function(name) {
  if (exists(name, mode = "function", inherits = TRUE)) {
    return(get(name, mode = "function", inherits = TRUE))
  }

  getFromNamespace(name, "twscrapeR")
}

with_envvars <- function(values, code) {
  vars <- names(values)
  old <- Sys.getenv(vars, unset = NA_character_)

  on.exit({
    for (var in vars) {
      if (is.na(old[[var]])) {
        Sys.unsetenv(var)
      } else {
        do.call(Sys.setenv, stats::setNames(as.list(old[[var]]), var))
      }
    }
  }, add = TRUE)

  for (var in vars) {
    if (is.na(values[[var]])) {
      Sys.unsetenv(var)
    } else {
      do.call(Sys.setenv, stats::setNames(as.list(values[[var]]), var))
    }
  }

  force(code)
}

test_that("search_tweets validates product before requiring configuration", {
  search_tweets_fn <- twscrapeR_test_fn("search_tweets")

  expect_error(
    search_tweets_fn("rstats", product = "Popular", progress = FALSE),
    "'arg' should be one of"
  )
})

test_that("search_tweets passes Latest product to twscrape by default", {
  python_code <- twscrapeR_test_fn(".search_tweets_python_code")

  code <- python_code("rstats", 10)

  expect_match(code, "api\\.search\\('rstats', limit=10, kv=\\{'product': 'Latest'\\}\\)")
})

test_that("search_tweets passes selected Top product to twscrape", {
  python_code <- twscrapeR_test_fn(".search_tweets_python_code")

  code <- python_code("rstats", 10, "Top")

  expect_match(code, "api\\.search\\('rstats', limit=10, kv=\\{'product': 'Top'\\}\\)")
})

test_that("search_tweets escapes query string literals", {
  python_code <- twscrapeR_test_fn(".search_tweets_python_code")

  code <- python_code("it's rstats", 10)

  expect_match(code, "api\\.search\\('it\\\\'s rstats', limit=10, kv=\\{'product': 'Latest'\\}\\)")
})

test_that("tweet IDs are extracted from supported inputs", {
  extract_tweet_ids <- twscrapeR_test_fn(".extract_tweet_ids")

  expect_identical(extract_tweet_ids(c(123, 456)), c("123", "456"))
  expect_identical(extract_tweet_ids(data.frame(tweet_id = c("a", "b"))), c("a", "b"))
  expect_identical(extract_tweet_ids(data.frame(id = c("x", "y"))), c("x", "y"))

  tweet <- structure(list(id = "999"), class = c("twscraper_tweet", "list"))
  expect_identical(extract_tweet_ids(tweet), "999")

  tweets <- list(list(id = "1"), list(id = 2), "3")
  expect_identical(extract_tweet_ids(tweets), c("1", "2", "3"))
})

test_that("tweet ID extraction rejects unsupported data frames", {
  extract_tweet_ids <- twscrapeR_test_fn(".extract_tweet_ids")

  expect_error(
    extract_tweet_ids(data.frame(other = "value")),
    "columna 'tweet_id' o 'id'"
  )
})

test_that("to_dataframe preserves source tweet ID for batch retweeters", {
  to_dataframe_fn <- twscrapeR_test_fn("to_dataframe")

  users <- list(
    list(id = "u1", username = "one", displayname = "One", description = "", followers_count = 1,
         following_count = 2, tweets_count = 3, verified = FALSE, created = "", location = "",
         url = "", profile_image_url = "", source_tweet_id = "t1"),
    list(id = "u2", username = "two", displayname = "Two", description = "", followers_count = 4,
         following_count = 5, tweets_count = 6, verified = TRUE, created = "", location = "",
         url = "", profile_image_url = "", source_tweet_id = "t2")
  )
  class(users) <- c("twscraper_users", "list")

  df <- to_dataframe_fn(users)

  expect_true("source_tweet_id" %in% names(df))
  expect_identical(as.character(df$source_tweet_id), c("t1", "t2"))
})

test_that("cookies can be read directly from env", {
  cookies_from_env <- twscrapeR_test_fn(".cookies_from_env")

  with_envvars(c(
    TEST_COOKIES = "auth_token=direct; ct0=direct",
    TEST_AUTH_TOKEN = "token",
    TEST_CT0 = "ct0"
  ), {
    expect_identical(cookies_from_env("TEST_"), "auth_token=direct; ct0=direct")
  })
})

test_that("cookies can be composed from token env vars", {
  cookies_from_env <- twscrapeR_test_fn(".cookies_from_env")

  with_envvars(c(
    TEST_COOKIES = NA_character_,
    TEST_AUTH_TOKEN = "token_value",
    TEST_CT0 = "ct0_value"
  ), {
    expect_identical(cookies_from_env("TEST_"), "auth_token=token_value; ct0=ct0_value")
  })
})

test_that("missing cookie env vars fail clearly", {
  cookies_from_env <- twscrapeR_test_fn(".cookies_from_env")

  with_envvars(c(
    TEST_COOKIES = NA_character_,
    TEST_AUTH_TOKEN = "token_value",
    TEST_CT0 = NA_character_
  ), {
    expect_error(cookies_from_env("TEST_"), "Faltan cookies")
  })
})

test_that("account env values are assembled without exposing secrets", {
  account_env_values <- twscrapeR_test_fn(".account_env_values")

  with_envvars(c(
    TEST_USERNAME = "usuario",
    TEST_PASSWORD = "password",
    TEST_EMAIL = "email@example.com",
    TEST_EMAIL_PASSWORD = "email_password",
    TEST_AUTH_TOKEN = "token_value",
    TEST_CT0 = "ct0_value",
    TEST_COOKIES = NA_character_
  ), {
    account <- account_env_values("TEST_")

    expect_identical(account$username, "usuario")
    expect_identical(account$email, "email@example.com")
    expect_identical(account$cookies, "auth_token=token_value; ct0=ct0_value")
  })
})

test_that("Python string literals escape unsafe characters", {
  python_string_literal <- twscrapeR_test_fn(".python_string_literal")

  expect_identical(python_string_literal("simple"), "'simple'")
  expect_identical(python_string_literal("a'b"), "'a\\'b'")
  expect_identical(python_string_literal("a\\b"), "'a\\\\b'")
  expect_identical(python_string_literal("a\nb"), "'a\\nb'")
  expect_error(python_string_literal(NA_character_), "string no vacío")
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping.R
\name{get_user}
\alias{get_user}
\title{Obtener Informacion de Usuario}
\usage{
get_user(username, progress = TRUE)
}
\arguments{
\item{username}{Nombre de usuario (sin @)}

\item{progress}{Mostrar progreso (default: TRUE)}
}
\value{
Lista con informacion del usuario
}
\description{
Obtiene informacion detallada de un usuario
}
\examples{
\donttest{
if (check_setup()) {
  user <- get_user("hadleywickham")
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{as.data.frame.twscraper_user}
\alias{as.data.frame.twscraper_user}
\title{Convertir Usuario a Data Frame}
\usage{
\method{as.data.frame}{twscraper_user}(x, ...)
}
\arguments{
\item{x}{Usuario de get_user()}

\item{...}{Argumentos adicionales}
}
\value{
data.frame
}
\description{
Convierte un usuario individual a data.frame de una fila
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping.R
\name{search_tweets}
\alias{search_tweets}
\title{Buscar Tweets}
\usage{
search_tweets(query, n = 100, progress = TRUE, product = c("Latest", "Top"))
}
\arguments{
\item{query}{Consulta de busqueda (ej: "rstats", "#datascience", "from:usuario")}

\item{n}{Numero maximo de tweets a retornar (default: 100)}

\item{progress}{Mostrar barra de progreso (default: TRUE)}

\item{product}{Tipo de busqueda: "Latest" para tweets recientes o "Top" para tweets destacados (default: "Latest")}
}
\value{
Lista de tweets
}
\description{
Busca tweets usando una consulta
}
\examples{
\donttest{
if (check_setup()) {
  tweets <- search_tweets("rstats", n = 50)
  top_tweets <- search_tweets("rstats", n = 50, product = "Top")
  df <- to_dataframe(tweets)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{sort_tweets}
\alias{sort_tweets}
\title{Ordenar Tweets}
\usage{
sort_tweets(tweets, by = "date", decreasing = TRUE)
}
\arguments{
\item{tweets}{Lista de tweets}

\item{by}{Campo por el cual ordenar ("date", "like_count", "retweet_count", "views_count")}

\item{decreasing}{Orden descendente (default: TRUE)}
}
\value{
Lista ordenada de tweets
}
\description{
Ordena tweets por algun criterio
}
\examples{
\donttest{
tweets <- list(
  list(id = "1", date = as.POSIXct("2026-01-01"), like_count = 2),
  list(id = "2", date = as.POSIXct("2026-02-01"), like_count = 10)
)
class(tweets) <- c("twscraper_tweets", "list")
top_tweets <- sort_tweets(tweets, by = "like_count")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accounts.R
\name{list_accounts}
\alias{list_accounts}
\title{Listar Cuentas}
\usage{
list_accounts(db_file = "accounts.db", verbose = TRUE)
}
\arguments{
\item{db_file}{Archivo de base de datos (default: "accounts.db")}

\item{verbose}{Si `TRUE`, emite mensajes informativos con `message()`/`warning()`; puede silenciarse con `suppressMessages()`/`suppressWarnings()`.}
}
\value{
Invisibly returns a list of configured accounts. Each account contains `username`, `email`, `active`, and `locks`; an empty list indicates no accounts or an error.
}
\description{
Lista todas las cuentas configuradas
}
\examples{
\donttest{
if (check_setup()) {
  accounts <- list_accounts()
  print(accounts)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup.R
\name{check_setup}
\alias{check_setup}
\title{Verificar si twscrapeR esta configurado}
\usage{
check_setup(...)
}
\arguments{
\item{...}{Ignorado; reservado para compatibilidad futura.}
}
\value{
`TRUE` si twscrapeR esta configurado en la sesion actual; `FALSE` en caso contrario.
}
\description{
Verifica si Python y twscrape estan configurados en la sesion actual.
}
\examples{
check_setup()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{to_dataframe}
\alias{to_dataframe}
\title{Convertir Tweets o Usuarios a Data Frame}
\usage{
to_dataframe(x)
}
\arguments{
\item{x}{Lista de tweets o usuarios}
}
\value{
data.frame o tibble
}
\description{
Convierte una lista de tweets o usuarios a data.frame usando purrr::map_dfr
}
\examples{
\donttest{
tweets <- list(
  list(
    id = "1", date = as.POSIXct("2026-01-01"), text = "Hello R",
    username = "rstats", user_displayname = "R Stats", user_id = "10",
    reply_count = 0, retweet_count = 1, like_count = 2, quote_count = 0,
    views_count = 100, lang = "en", url = "https://example.com/1",
    user_followers = 1000, user_verified = FALSE
  )
)
class(tweets) <- c("twscraper_tweets", "list")
df <- to_dataframe(tweets)

users <- list(list(
  id = "10", username = "rstats", displayname = "R Stats",
  description = "Example user", followers_count = 1000,
  following_count = 50, tweets_count = 200, verified = FALSE,
  created = "2026-01-01", location = "", url = "",
  profile_image_url = ""
))
class(users) <- c("twscraper_users", "list")
users_df <- to_dataframe(users)

users[[1]]$source_tweet_id <- "1"
retweeters_df <- to_dataframe(users)  # incluye source_tweet_id
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accounts.R
\name{add_account}
\alias{add_account}
\title{Agregar Cuenta de Twitter}
\usage{
add_account(
  username,
  password,
  email,
  email_password,
  cookies,
  db_file = "accounts.db",
  verbose = TRUE
)
}
\arguments{
\item{username}{Nombre de usuario}

\item{password}{Contrasena}

\item{email}{Email de la cuenta}

\item{email_password}{Contrasena del email}

\item{cookies}{Cookies de sesion (OBLIGATORIO - formato: "auth_token=...; ct0=...")}

\item{db_file}{Archivo de base de datos (default: "accounts.db")}

\item{verbose}{Si `TRUE`, emite mensajes informativos con `message()`/`warning()`; puede silenciarse con `suppressMessages()`/`suppressWarnings()`.}
}
\value{
Invisibly returns `TRUE` if the account was added and is active; invisibly returns `FALSE` if it was not added, already existed, or could not be activated.
}
\description{
Agrega una cuenta de Twitter con cookies y la activa automaticamente. IMPORTANTE: Las cookies son obligatorias para activar la cuenta correctamente.
}
\examples{
\donttest{
if (check_setup() && interactive()) {
  # add_account(
  #   username = "your_username",
  #   password = "your_password",
  #   email = "you@example.com",
  #   email_password = "your_email_password",
  #   cookies = "auth_token=...; ct0=..."
  # )
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping.R
\name{search_hashtag}
\alias{search_hashtag}
\title{Buscar por Hashtag}
\usage{
search_hashtag(hashtag, n = 100, progress = TRUE)
}
\arguments{
\item{hashtag}{Hashtag (con o sin #)}

\item{n}{Numero maximo de tweets (default: 100)}

\item{progress}{Mostrar progreso (default: TRUE)}
}
\value{
Lista de tweets
}
\description{
Busca tweets que contengan un hashtag especifico
}
\examples{
\donttest{
if (check_setup()) {
  tweets <- search_hashtag("rstats", n = 50)
  tweets <- search_hashtag("#datascience", n = 100)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping_advanced.R
\name{user_tweets_and_replies}
\alias{user_tweets_and_replies}
\title{Obtener Tweets y Respuestas de un Usuario}
\usage{
user_tweets_and_replies(username, n = 100, progress = TRUE)
}
\arguments{
\item{username}{Nombre de usuario (sin @)}

\item{n}{Numero maximo de tweets (default: 100)}

\item{progress}{Mostrar progreso (default: TRUE)}
}
\value{
Lista de tweets
}
\description{
Obtiene tweets y respuestas de un usuario (timeline completo)
}
\examples{
\donttest{
if (check_setup()) {
  all_tweets <- user_tweets_and_replies("elonmusk", n = 100)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{print.twscraper_tweet}
\alias{print.twscraper_tweet}
\title{Metodo Print para Tweet Individual}
\usage{
\method{print}{twscraper_tweet}(x, ...)
}
\arguments{
\item{x}{Informacion de un tweet}

\item{...}{Argumentos adicionales}
}
\value{
Invisibly returns `x`, a `twscraper_tweet` list with tweet metadata, author fields, engagement counts, language, and URL.
}
\description{
Imprime informacion de un tweet individual
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping_advanced.R
\name{users_to_dataframe}
\alias{users_to_dataframe}
\title{Convertir Usuarios a Dataframe (alias de to_dataframe)}
\usage{
users_to_dataframe(users)
}
\arguments{
\item{users}{Lista de usuarios}
}
\value{
data.frame con informacion de usuarios
}
\description{
Alias interno de to_dataframe() - usar to_dataframe() directamente
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping_advanced.R
\name{get_retweeters}
\alias{get_retweeters}
\title{Obtener Usuarios que Retuitearon}
\usage{
get_retweeters(tweet_id, n = 100, progress = TRUE)
}
\arguments{
\item{tweet_id}{ID del tweet}

\item{n}{Numero maximo de usuarios (default: 100)}

\item{progress}{Mostrar progreso (default: TRUE)}
}
\value{
Lista de usuarios
}
\description{
Obtiene la lista de usuarios que retuitearon un tweet especifico
}
\examples{
\donttest{
if (check_setup()) {
  retweeters <- get_retweeters(1234567890, n = 100)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{print.twscraper_user}
\alias{print.twscraper_user}
\title{Metodo Print para Usuario}
\usage{
\method{print}{twscraper_user}(x, ...)
}
\arguments{
\item{x}{Informacion de usuario}

\item{...}{Argumentos adicionales}
}
\value{
Invisibly returns `x`, a `twscraper_user` list with fields such as `id`, `username`, `displayname`, `description`, follower/following counts, verification status, location, URL, and profile image URL.
}
\description{
Imprime informacion de usuario
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping_advanced.R
\name{get_retweeters_batch}
\alias{get_retweeters_batch}
\title{Obtener Retweeters para Varios Tweets}
\usage{
get_retweeters_batch(tweets, n = 100, progress = TRUE, flatten = TRUE)
}
\arguments{
\item{tweets}{Vector de IDs, lista de tweets `twscraper_tweets`, tweet individual o data.frame con columna `id`/`tweet_id`.}

\item{n}{Numero maximo de usuarios por tweet (default: 100)}

\item{progress}{Mostrar progreso (default: TRUE)}

\item{flatten}{Si es TRUE, devuelve una lista plana de usuarios con columna `source_tweet_id` para usar con `to_dataframe()`. Si es FALSE, devuelve una lista agrupada por tweet.}
}
\value{
Lista de usuarios con `source_tweet_id` o lista agrupada por tweet
}
\description{
Obtiene usuarios que retuitearon varios tweets en una sola llamada de R.
}
\examples{
\donttest{
if (check_setup()) {
  tweets <- search_tweets("rstats", n = 10)
  retweeters <- get_retweeters_batch(tweets, n = 50)
  retweeters_df <- to_dataframe(retweeters)

  retweeters_by_tweet <- get_retweeters_batch(tweets, n = 50, flatten = FALSE)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{save_csv}
\alias{save_csv}
\title{Guardar Tweets como CSV}
\usage{
save_csv(tweets, file)
}
\arguments{
\item{tweets}{Lista de tweets}

\item{file}{Nombre del archivo}
}
\value{
Invisibly returns the output file path. The CSV contains one row per tweet/user-like item after conversion with `to_dataframe()`.
}
\description{
Guarda tweets en formato CSV
}
\examples{
\donttest{
tweets <- list(list(id = "1", text = "Hello R", username = "rstats"))
save_csv(tweets, tempfile(fileext = ".csv"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping_advanced.R
\name{tweet_details}
\alias{tweet_details}
\title{Obtener Detalles de un Tweet}
\usage{
tweet_details(tweet_id, progress = TRUE)
}
\arguments{
\item{tweet_id}{ID del tweet (numero o string)}

\item{progress}{Mostrar progreso (default: TRUE)}
}
\value{
Lista con informacion del tweet o NULL si no se encuentra
}
\description{
Obtiene informacion detallada de un tweet especifico por su ID
}
\examples{
\donttest{
if (check_setup()) {
  tweet <- tweet_details(1234567890)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup.R
\name{setup_twscraper}
\alias{setup_twscraper}
\title{Setup twscrapeR}
\usage{
setup_twscraper(python_path = NULL, install_python = TRUE, ask = TRUE)
}
\arguments{
\item{python_path}{Ruta a Python (opcional, se detecta automaticamente)}

\item{install_python}{Deprecated compatibility argument. Ignored; installation must be performed by the user outside package functions.}

\item{ask}{Deprecated compatibility argument. Ignored; retained to avoid breaking existing calls.}
}
\value{
Invisibly returns `TRUE` when Python >= 3.10 and the Python `twscrape` module are available and configured. Invisibly returns `FALSE` otherwise, after printing installation instructions.
}
\description{
Checks that Python and the Python `twscrape` package are available and configures the active reticulate session. It does not install Python or Python packages automatically.
}
\examples{
\donttest{
if (interactive()) {
  setup_twscraper()
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{print.twscraper_users}
\alias{print.twscraper_users}
\title{Metodo Print para Lista de Usuarios}
\usage{
\method{print}{twscraper_users}(x, ...)
}
\arguments{
\item{x}{Lista de usuarios}

\item{...}{Argumentos adicionales}
}
\value{
Invisibly returns `x`, a `twscraper_users` list. Each element represents one user with profile metadata and account statistics.
}
\description{
Imprime resumen de usuarios
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accounts.R
\name{delete_account}
\alias{delete_account}
\title{Eliminar Cuenta}
\usage{
delete_account(username, db_file = "accounts.db", verbose = TRUE)
}
\arguments{
\item{username}{Nombre de usuario a eliminar}

\item{db_file}{Archivo de base de datos (default: "accounts.db")}

\item{verbose}{Si `TRUE`, emite mensajes informativos con `message()`/`warning()`; puede silenciarse con `suppressMessages()`/`suppressWarnings()`.}
}
\value{
Invisibly returns `TRUE` if the account was deleted; invisibly returns `FALSE` otherwise.
}
\description{
Elimina una cuenta de la base de datos
}
\examples{
\donttest{
if (check_setup() && interactive()) {
  # delete_account("your_username")
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping.R
\name{search_mentions}
\alias{search_mentions}
\title{Buscar Menciones de un Usuario}
\usage{
search_mentions(username, n = 100, progress = TRUE)
}
\arguments{
\item{username}{Nombre de usuario (sin @)}

\item{n}{Numero maximo de tweets (default: 100)}

\item{progress}{Mostrar progreso (default: TRUE)}
}
\value{
Lista de tweets
}
\description{
Busca tweets que mencionen a un usuario especifico
}
\examples{
\donttest{
if (check_setup()) {
  mentions <- search_mentions("hadleywickham", n = 50)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{filter_by_date}
\alias{filter_by_date}
\title{Filtrar Tweets por Fecha}
\usage{
filter_by_date(tweets, from = NULL, to = NULL)
}
\arguments{
\item{tweets}{Lista de tweets}

\item{from}{Fecha inicial (POSIXct o character)}

\item{to}{Fecha final (POSIXct o character)}
}
\value{
Lista filtrada de tweets
}
\description{
Filtra tweets por rango de fechas
}
\examples{
\donttest{
tweets <- list(
  list(id = "1", date = as.POSIXct("2026-01-01")),
  list(id = "2", date = as.POSIXct("2026-02-01"))
)
class(tweets) <- c("twscraper_tweets", "list")
tweets_recent <- filter_by_date(tweets, from = "2025-10-01")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{save_json}
\alias{save_json}
\title{Guardar Tweets como JSON}
\usage{
save_json(tweets, file)
}
\arguments{
\item{tweets}{Lista de tweets}

\item{file}{Nombre del archivo}
}
\value{
Invisibly returns the output file path. The JSON file contains the list structure supplied in `tweets`, written with `jsonlite::write_json()`.
}
\description{
Guarda tweets en formato JSON
}
\examples{
\donttest{
tweets <- list(list(id = "1", text = "Hello R", username = "rstats"))
save_json(tweets, tempfile(fileext = ".json"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accounts.R
\name{add_account_from_env}
\alias{add_account_from_env}
\title{Agregar Cuenta desde Variables de Entorno}
\usage{
add_account_from_env(prefix = "TWS_", db_file = "accounts.db", verbose = TRUE)
}
\arguments{
\item{prefix}{Prefijo de las variables de entorno (default: "TWS_")}

\item{db_file}{Archivo de base de datos (default: "accounts.db")}

\item{verbose}{Si `TRUE`, emite mensajes informativos con `message()`/`warning()`; puede silenciarse con `suppressMessages()`/`suppressWarnings()`.}
}
\value{
`TRUE` si la cuenta se agrego y activo; `FALSE` si se agrego pero no quedo activa o hubo error
}
\description{
Lee credenciales desde `.Renviron` o variables de entorno y agrega la cuenta sin exponer secretos en el codigo.
}
\examples{
\donttest{
# En .Renviron:
# TWS_USERNAME='mi_usuario'
# TWS_PASSWORD='mi_password'
# TWS_EMAIL='mi@email.com'
# TWS_EMAIL_PASSWORD='email_pass'
# TWS_AUTH_TOKEN='auth_token_del_navegador'
# TWS_CT0='ct0_del_navegador'

if (check_setup() && interactive()) {
  add_account_from_env()

  # Para una segunda cuenta, usa otro prefijo:
  # TWS2_USERNAME='otra_cuenta'
  # TWS2_PASSWORD='otra_password'
  # ...
  add_account_from_env(prefix = "TWS2_")
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup.R
\name{ensure_configured}
\alias{ensure_configured}
\title{Asegurar que twscrapeR esta configurado}
\usage{
ensure_configured()
}
\description{
Uso interno - verifica configuracion antes de operaciones
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{filter_by_lang}
\alias{filter_by_lang}
\title{Filtrar Tweets por Idioma}
\usage{
filter_by_lang(tweets, lang)
}
\arguments{
\item{tweets}{Lista de tweets}

\item{lang}{Codigo de idioma (ej: "es", "en", "pt")}
}
\value{
Lista filtrada de tweets
}
\description{
Filtra tweets por idioma
}
\examples{
\donttest{
tweets <- list(
  list(id = "1", lang = "es"),
  list(id = "2", lang = "en")
)
class(tweets) <- c("twscraper_tweets", "list")
tweets_es <- filter_by_lang(tweets, "es")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping_advanced.R
\name{user_media}
\alias{user_media}
\title{Obtener Tweets con Media de un Usuario}
\usage{
user_media(username, n = 100, progress = TRUE)
}
\arguments{
\item{username}{Nombre de usuario (sin @)}

\item{n}{Numero maximo de tweets (default: 100)}

\item{progress}{Mostrar progreso (default: TRUE)}
}
\value{
Lista de tweets
}
\description{
Obtiene solo los tweets que contienen imagenes o videos
}
\examples{
\donttest{
if (check_setup()) {
  media_tweets <- user_media("elonmusk", n = 100)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{print.twscraper_tweets}
\alias{print.twscraper_tweets}
\title{Metodo Print para Tweets}
\usage{
\method{print}{twscraper_tweets}(x, ...)
}
\arguments{
\item{x}{Lista de tweets}

\item{...}{Argumentos adicionales}
}
\value{
Invisibly returns `x`, a `twscraper_tweets` list. Each element represents one tweet with fields such as `id`, `date`, `text`, `username`, engagement counts, language, and URL.
}
\description{
Imprime resumen de tweets
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping_advanced.R
\name{verified_followers}
\alias{verified_followers}
\title{Obtener Seguidores Verificados}
\usage{
verified_followers(username, n = 100, progress = TRUE)
}
\arguments{
\item{username}{Nombre de usuario (sin @)}

\item{n}{Numero maximo de seguidores (default: 100)}

\item{progress}{Mostrar progreso (default: TRUE)}
}
\value{
Lista de usuarios verificados
}
\description{
Obtiene solo los seguidores que tienen cuenta verificada
}
\examples{
\donttest{
if (check_setup()) {
  verified <- verified_followers("elonmusk", n = 100)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping_advanced.R
\name{tweet_replies}
\alias{tweet_replies}
\title{Obtener Respuestas a un Tweet}
\usage{
tweet_replies(tweet_id, n = 100, progress = TRUE)
}
\arguments{
\item{tweet_id}{ID del tweet}

\item{n}{Numero maximo de respuestas (default: 100)}

\item{progress}{Mostrar progreso (default: TRUE)}
}
\value{
Lista de tweets (respuestas)
}
\description{
Obtiene las respuestas (replies) a un tweet especifico
}
\examples{
\donttest{
if (check_setup()) {
  replies <- tweet_replies(1234567890, n = 50)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup.R
\docType{data}
\name{.twscraper_env}
\alias{.twscraper_env}
\title{Entorno global para almacenar configuracion}
\format{
An object of class \code{environment} of length 3.
}
\usage{
.twscraper_env
}
\description{
Entorno global para almacenar configuracion
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scraping_advanced.R
\name{get_following}
\alias{get_following}
\title{Obtener Usuarios Seguidos}
\usage{
get_following(username, n = 100, progress = TRUE)
}
\arguments{
\item{username}{Nombre de usuario (sin @)}

\item{n}{Numero maximo de usuarios (default: 100)}

\item{progress}{Mostrar progreso (default: TRUE)}
}
\value{
Lista de usuarios
}
\description{
Obtiene la lista de usuarios que sigue un usuario
}
\examples{
\donttest{
if (check_setup()) {
  following <- get_following("elonmusk", n = 100)
}
}
}
